/*******************************************************************************
 * Copyright (c) 2006 Tom Seidel, Spirit Link GmbH
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * 
 * Contributors:
 *     Tom Seidel - initial API and implementation
 *******************************************************************************/
package de.spiritlink.richhtml4eclipse.example.views;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;

import de.spiritlink.richhtml4eclipse.example.actions.BackColorAction;
import de.spiritlink.richhtml4eclipse.example.actions.BoldAction;
import de.spiritlink.richhtml4eclipse.example.actions.BulletListAction;
import de.spiritlink.richhtml4eclipse.example.actions.CleanupAction;
import de.spiritlink.richhtml4eclipse.example.actions.DeleteColumnAction;
import de.spiritlink.richhtml4eclipse.example.actions.DeleteRowAction;
import de.spiritlink.richhtml4eclipse.example.actions.ForegroundAction;
import de.spiritlink.richhtml4eclipse.example.actions.HrAction;
import de.spiritlink.richhtml4eclipse.example.actions.IndentAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertColumnAfterAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertColumnBeforeAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertEditAnchorAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertEditImageAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertEditLinkAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertEditTableAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertLayerAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertNonBreakingWhitespace;
import de.spiritlink.richhtml4eclipse.example.actions.InsertRowAfterAction;
import de.spiritlink.richhtml4eclipse.example.actions.InsertRowBeforeAction;
import de.spiritlink.richhtml4eclipse.example.actions.ItalicAction;
import de.spiritlink.richhtml4eclipse.example.actions.JustifyCenterAction;
import de.spiritlink.richhtml4eclipse.example.actions.JustifyFullAction;
import de.spiritlink.richhtml4eclipse.example.actions.JustifyLeftAction;
import de.spiritlink.richhtml4eclipse.example.actions.JustifyRightAction;
import de.spiritlink.richhtml4eclipse.example.actions.MakeLayerAbsoluteAction;
import de.spiritlink.richhtml4eclipse.example.actions.MoveLayerBackwardAction;
import de.spiritlink.richhtml4eclipse.example.actions.MoveLayerForwardAction;
import de.spiritlink.richhtml4eclipse.example.actions.NumListAction;
import de.spiritlink.richhtml4eclipse.example.actions.OutdentAction;
import de.spiritlink.richhtml4eclipse.example.actions.RedoAction;
import de.spiritlink.richhtml4eclipse.example.actions.RemoveFormatAction;
import de.spiritlink.richhtml4eclipse.example.actions.SelectFontAction;
import de.spiritlink.richhtml4eclipse.example.actions.SelectFontSizeAction;
import de.spiritlink.richhtml4eclipse.example.actions.SelectFormatAction;
import de.spiritlink.richhtml4eclipse.example.actions.SetHtmlAction;
import de.spiritlink.richhtml4eclipse.example.actions.StrikeThroughAction;
import de.spiritlink.richhtml4eclipse.example.actions.SubAction;
import de.spiritlink.richhtml4eclipse.example.actions.SupAction;
import de.spiritlink.richhtml4eclipse.example.actions.ToggleVisualAidAction;
import de.spiritlink.richhtml4eclipse.example.actions.UnderLineAction;
import de.spiritlink.richhtml4eclipse.example.actions.UndoAction;
import de.spiritlink.richhtml4eclipse.example.actions.UnlinkAction;
import de.spiritlink.richhtml4eclipse.widgets.HtmlComposer;
import de.spiritlink.richhtml4eclipse.widgets.JavaScriptCommands;

public class HTMLView {

	private HtmlComposer composer;

	/**
	 * The constructor.
	 */
	public HTMLView(Composite com,String htmlContent) {
		createPartControl(com,htmlContent);
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	private void createPartControl(final Composite parent,String htmlContent) {
		
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;
/*
 detailsText = new Text(parent, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
 | SWT.BORDER);*/
		final Composite comp = new Composite(parent, SWT.MULTI | SWT.BORDER);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		//gridLayout.horizontalSpacing = 5;
		comp.setLayout(gridLayout);
		comp.setLayoutData(gridData);

		CoolBar coolbar = new CoolBar(comp, SWT.NONE);
		GridData gridData1 = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
		//    gridData1.heightHint = 100;
		//   gridData1.grabExcessHorizontalSpace = true;
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.horizontalSpan = 5;
		//  gridData1.grabExcessVerticalSpace = true;
		/*  GridData gd = new GridData(SWT.FILL,SWT.BEGINNING, true, false);
		  gd.widthHint = 100;*/
		coolbar.setLayoutData(gridData1);

		coolbar.addListener(SWT.Resize, new Listener() {
			public void handleEvent(Event event) {
				comp.getShell().layout();
			}
		});

		composer = new HtmlComposer(comp, SWT.NONE);
		composer.setLayoutData(gridData);
		composer.execute(JavaScriptCommands.SET_HTML(htmlContent));

		final Composite comboComp = new Composite(coolbar, SWT.NONE);
		comboComp.setLayout(new GridLayout(3, false));

		final Combo formatCombo = new Combo(comboComp, SWT.SINGLE);

		formatCombo.setItems(SelectFormatAction.formatMappings.values()
				.toArray(new String[0]));
		formatCombo.add("--[Format]--", 0);
		formatCombo.select(0);
		formatCombo.addSelectionListener(new SelectionAdapter() {
			private Action action = new SelectFormatAction(composer,
					formatCombo);

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				this.action.run();
			}
		});
		Point formatTextSize = formatCombo
				.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		formatTextSize = formatCombo.computeSize(formatTextSize.x,
				formatTextSize.y);

		final Combo fontCombo = new Combo(comboComp, SWT.SINGLE);

		fontCombo.setItems(getFontList());
		fontCombo.add("--[Font]--", 0);
		fontCombo.select(0);
		fontCombo.addSelectionListener(new SelectionAdapter() {
			private Action action = new SelectFontAction(composer, fontCombo);

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				this.action.run();
			}
		});
		Point fontTextSize = fontCombo.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		fontTextSize = fontCombo.computeSize(fontTextSize.x, fontTextSize.y);

		final Combo fontsizeCombo = new Combo(comboComp, SWT.SINGLE);
		fontsizeCombo.setItems(new String[] { "--[Size]--", "1", "2", "3", "4",
				"5", "6" });
		fontsizeCombo.select(0);
		fontsizeCombo.addSelectionListener(new SelectionAdapter() {
			private Action action = new SelectFontSizeAction(composer,
					fontsizeCombo);

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				this.action.run();
			}
		});
		CoolItem comboItem = new CoolItem(coolbar, SWT.NONE);
		comboItem.setControl(comboComp);

		Point fontSizetextSize = fontsizeCombo.computeSize(SWT.DEFAULT,
				SWT.DEFAULT);
		fontSizetextSize = fontsizeCombo.computeSize(fontSizetextSize.x,
				fontSizetextSize.y);
		comboItem.setMinimumSize(formatTextSize.x + fontSizetextSize.x
				+ fontTextSize.x, formatTextSize.y + fontSizetextSize.y
				+ fontTextSize.y);
		comboItem.setPreferredSize(formatTextSize.x + fontSizetextSize.x
				+ fontTextSize.x, formatTextSize.y + fontSizetextSize.y
				+ fontTextSize.y);
		comboItem.setSize(formatTextSize.x + fontSizetextSize.x
				+ fontTextSize.x, formatTextSize.y + fontSizetextSize.y
				+ fontTextSize.y);

		ToolBar menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		ToolBarManager manager = new ToolBarManager(menu);
		CoolItem item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);

		manager.add(new BoldAction(composer));
		manager.add(new ItalicAction(composer));
		manager.add(new UnderLineAction(composer));
		manager.add(new StrikeThroughAction(composer));
		manager.add(new Separator());
		manager.add(new JustifyLeftAction(composer));
		manager.add(new JustifyCenterAction(composer));
		manager.add(new JustifyRightAction(composer));
		manager.add(new JustifyFullAction(composer));
		manager.add(new Separator());
		manager.add(new BulletListAction(composer));
		manager.add(new NumListAction(composer));
		manager.add(new Separator());
		manager.add(new OutdentAction(composer));
		manager.add(new IndentAction(composer));
		manager.add(new Separator());
		manager.add(new SubAction(composer));
		manager.add(new SupAction(composer));
		manager.update(true);

		menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		manager = new ToolBarManager(menu);
		item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);
		//For Hali
		
		manager.add(new SetHtmlAction(composer));
		manager.update(true);

		menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		manager = new ToolBarManager(menu);
		item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);

		manager.add(new InsertEditImageAction(composer));
		manager.add(new InsertEditAnchorAction(composer));
		manager.add(new InsertEditLinkAction(composer));
		manager.add(new UnlinkAction(composer));
		manager.add(new HrAction(composer));
		manager.add(new InsertNonBreakingWhitespace(composer));
		manager.add(new Separator());
		manager.add(new CleanupAction(composer));
		manager.add(new RemoveFormatAction(composer));
		manager.add(new ToggleVisualAidAction(composer));
		manager.add(new Separator());
		manager.add(new UndoAction(composer));
		manager.add(new RedoAction(composer));
		manager.update(true);

		menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		manager = new ToolBarManager(menu);
		item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);

		manager.add(new ForegroundAction(composer));
		manager.add(new BackColorAction(composer));
		manager.update(true);

		menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		manager = new ToolBarManager(menu);
		item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);

		manager.add(new InsertLayerAction(composer));
		manager.add(new MoveLayerBackwardAction(composer));
		manager.add(new MoveLayerForwardAction(composer));
		manager.add(new MakeLayerAbsoluteAction(composer));
		manager.update(true);
//        
		menu = new ToolBar(coolbar, SWT.HORIZONTAL | SWT.FLAT);
		manager = new ToolBarManager(menu);
		item = new CoolItem(coolbar, SWT.NONE);
		item.setControl(menu);

		manager.add(new InsertEditTableAction(composer));
		manager.add(new Separator());
		manager.add(new InsertRowBeforeAction(composer));
		manager.add(new InsertRowAfterAction(composer));
		manager.add(new DeleteRowAction(composer));
		manager.add(new Separator());
		manager.add(new InsertColumnBeforeAction(composer));
		manager.add(new InsertColumnAfterAction(composer));
		manager.add(new DeleteColumnAction(composer));

		manager.update(true);

//        /* Set the sizes after adding all cool items */
		CoolItem[] coolItems = coolbar.getItems();
		for (int i = 0; i < coolItems.length; i++) {
			CoolItem coolItem = coolItems[i];
			Control control = coolItem.getControl();
			Point size = control.computeSize(SWT.DEFAULT, SWT.DEFAULT);
			Point coolSize = coolItem.computeSize(size.x, size.y);
			if (control instanceof ToolBar) {
				ToolBar bar = (ToolBar) control;
				for (int j = 0, n = bar.getItemCount(); j < n; j++) {
					size.x += bar.getSize().x;
				}
			}
			coolItem.setMinimumSize(size);
			coolItem.setPreferredSize(coolSize);
			coolItem.setSize(coolSize);
		}

	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
//		viewer.getControl().setFocus();
	}

	private static String[] getFontList() {
		Set s = new HashSet();
		// Add names of all bitmap fonts.
		FontData[] fds = Display.getCurrent().getFontList(null, false);
		for (int i = 0; i < fds.length; ++i)
			s.add(fds[i].getName());
		// Add names of all scalable fonts.
		fds = Display.getCurrent().getFontList(null, true);
		for (int i = 0; i < fds.length; ++i)
			s.add(fds[i].getName());
		// Sort the result and print it.
		String[] answer = new String[s.size()];
		s.toArray(answer);
		Arrays.sort(answer);
		return answer;
	}

	public String getComposerConent() {
		return composer.getHtml();
	}

}