/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.main;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.ListenerList;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.Visitor;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;
import org.eclipse.samples.organizer.mails.filters.AccountFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilterByFolderID;
import org.eclipse.samples.organizer.models.Mail;

//singleton
public class MailManager {

	private Map<Integer, MailFolder> folderMap = new HashMap<Integer, MailFolder>();

	private static MailManager instance = null;

	private ListenerList contentChangeListenerList = new ListenerList();

	private static List<MailAccount> accounts = new LinkedList<MailAccount>();

	private Backend backend;
	private List<Backend> backends;
	
	private MailFolder currentFolder;
	
	private MailAccount currentaccount;

	private static boolean accountExist = false;

	private Backend getDefaultBackend() {
		backends = getBackends();
		assert backends != null;
		assert backends.size() > 0;
		
		Backend preferred = OrganizerManager.instance.getPreferredMailBackend();
		if (preferred == null) return backends.get(1);
		return preferred;
	}
	private List<Backend> getBackends() {
		return OrganizerManager.instance.getBackends();
	}
	private MailManager() {
			MailAccount[] accs = new MailAccount[0];
			try {
				accs = (MailAccount[]) OrganizerManager.instance
				.find(AccountFilter.instance);
			} catch (BackendException e) {
				e.printStackTrace();
			}
			//If there is no account, create a new one
			if(accs.length==0){
				
			}
			accounts = Arrays.asList(accs);
			for (int i = 0; i < accounts.size(); i++) {
				for (int j = 0; j < ((accounts.get(i)).getFolder())
						.getChildren().length; j++) {
				  if(i==j&&j==0)
					  this.currentaccount = accounts.get(i);
					folderMap.put(((MailFolder) ((accounts.get(i)).getFolder())
							.getChildren()[j]).getFolderID(),
							((MailFolder) ((accounts.get(i)).getFolder())
									.getChildren()[j]));
				}
		}
	}
	public static MailManager getInstance() {
		if (instance == null) {
			instance = new MailManager();
		}

		return instance;
	}

	public synchronized List<MailAccount> getAccounts() {
		return accounts;
	}

	public void addContentChangeListener(
			ContentChangeListener contentChangeListener) {
		contentChangeListenerList.add(contentChangeListener);
	}

	private void fireContentChangeEvent(ContentChangeEvent event) {
		for (Object object : contentChangeListenerList.getListeners()) {
			ContentChangeListener listener = (ContentChangeListener) object;
			listener.contentChanged(event);
		}
	}

	public void addSingleMailToDatabase(Mail mail) {
		backend = getDefaultBackend();
		OrganizerManager.instance.setPreferredMailBackend(backend);
		List<Mail> mails = Collections.singletonList(mail);
		try {
			OrganizerManager.instance.add(backend, mails);
		} catch (BackendException e) {
			e.printStackTrace();
		}
	}
	
	public void deleteMailsFromDatabase(List<Mail> mails) {
		this.backend = getDefaultBackend();
		try {
			OrganizerManager.instance.delete(backend, mails);
		} catch (BackendException e) {
			e.printStackTrace();
	}
		fireContentChangeEvent(new ContentChangeEvent());
	}

	public void addMultiMailsToDatabase(List<Mail> mails) {
		try {
			OrganizerManager.instance.add(backend, mails);
		} catch (BackendException e) {
			e.printStackTrace();
		}
	}
	public void fetchMailUIDs(final List<String> uidList, int folderID) throws BackendException {
		MailFilterByFolderID.instance.setFolderID(folderID); //TODO For fodlerID
		OrganizerManager.instance.find(MailFilterByFolderID.instance, new Visitor() {
			public void visit(Object object) {
				Mail mail = (Mail) object;
				uidList.add(mail.getUid());
			}
		});
	}
	
	public void fetchMails(final List<Mail> mailList, int folderID) throws BackendException {
		MailFilterByFolderID.instance.setFolderID(folderID); //TODO For fodlerID
		OrganizerManager.instance.find(MailFilterByFolderID.instance, new Visitor() {
			public void visit(Object object) {
				mailList.add((Mail) object);
			}
		});
	}
	
	public ListenerList getContentChangeListenerList() {
		return contentChangeListenerList;
	}

	public void setContentChangeListenerList(
			ListenerList contentChangeListenerList) {
		this.contentChangeListenerList = contentChangeListenerList;
	}

	public MailFolder getCurrentFolder() {
		return currentFolder;
	}

	public synchronized void setCurrentFolder(MailFolder currentFolder) {
		this.currentFolder = currentFolder;
		this.currentaccount = getAccountFromID(currentFolder.getAccountID());
	}

	public MailFolder getMailFolderByID(int folderID) {
		return folderMap.get(folderID);
	}


	public boolean isAccountNone() {
		return !accountExist;
	}

	public String[] getAccountsName() {
		if (accountExist) {
			List<String> accountName = new ArrayList<String>();
			String temp[] = new String[getAccounts().size()];
			for (int i = 0; i < getAccounts().size(); i++) {
				accountName.add(accounts.get(i).getAccountName());
			}
			accountName.toArray(temp);
			return temp;
		}
		return null;
	}
	public int[] getAccountsID() {
		if (accountExist) {
			int temp[] = new int[getAccounts().size()];
			for (int i = 0; i < getAccounts().size(); i++) {
				temp[i] =accounts.get(i).getAccountID();
			}
			return temp;
		}
		return null;
	}

	public MailAccount getAccountFromName(String name) {
		for (int i = 0; i < getAccounts().size(); i++) {
			if (name.equalsIgnoreCase(accounts.get(i).getAccountName())) {
				return accounts.get(i);
			}
		}
		return null;
	}
	
	public synchronized MailAccount getAccountFromID(int accountID) {
		for (int i = 0; i < accounts.size(); i++) {
			if (accountID== accounts.get(i).getAccountID()) {
				return accounts.get(i);
			}
		}
		return null;
	}

	public int getFolderIDByString(String folderName) {
		
		if (currentFolder == null) {
			for (Iterator<MailFolder> i = folderMap.values().iterator(); i
					.hasNext();) {
				MailFolder mf = i.next();
				if (mf.getFolderName().equalsIgnoreCase(folderName)
						&& mf.getAccountID()==currentaccount.getAccountID())
					return mf.getFolderID();
			}
		}
		int currentfolderID = this.getCurrentFolder().getFolderID();
		for (Iterator<MailFolder> i = folderMap.values().iterator(); i
				.hasNext();) {
			MailFolder mf = i.next();
			if (mf.getFolderName().equalsIgnoreCase(folderName)
					&& mf.getAccountID()==folderMap.get(currentfolderID).getAccountID())
				return mf.getFolderID();
		}
		return currentfolderID;
	}

	/**
	 * Make sure that mail account should be set first.
	 * @param mail
	 * @param folderName
	 */
	public void changeFolderIDByName(Mail mail, String folderName) {
		for (Iterator it = folderMap.entrySet().iterator(); it.hasNext();) {
			Map.Entry e = (Map.Entry) it.next();
			if (((MailFolder) e.getValue()).getAccountID()==
					mail.getAccountID()
					&& (((MailFolder) e.getValue()).getFolderName()
							.equalsIgnoreCase(folderName))) {
				mail.setFolderID(Integer.parseInt(e.getKey().toString()));
				break;
			}
		}
	}

	public void changeFolderIDByNameAndSave(Mail mail, String folderName) {
		for (Iterator it = folderMap.entrySet().iterator(); it.hasNext();) {
			Map.Entry e = (Map.Entry) it.next();
			if (((MailFolder) e.getValue()).getAccountID()==
					mail.getAccountID()
					&& (((MailFolder) e.getValue()).getFolderName()
							.equalsIgnoreCase(folderName))) {
				mail.setFolderIDToDataBase(Integer.parseInt(e.getKey()
						.toString()));
				break;
			}
		}
	}

	public MailAccount getCurrentaccount() {
		return currentaccount;
	}

	public void setCurrentAccountFromID(int id) {
		this.currentaccount = getAccountFromID(id);
	}
}
