/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.models;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Mail {
	public enum Priority {
		LOW, NORMAL, HIGH
	};
	
	public enum MailFormat {
		PLAIN, HTML
	}
	
	public enum MailFlag {
		ANSWERED,DRAFT,FLAGGED,RECENT,SEEN,DELETED
	}
	
	public enum MailPrivateTag {
		NONE,IMPORTANT,COMPANY,FRIEND,FUNNY,HOBBY,FAMILY,BUSINESS,TECHONOLOGY
	}
    // identify mail id, it can be used to open mail from local files
	int id;
	String uid;
	int size;
	String date;
	String sender;
	String receivers;
	String ccers;
	String bccers;
	Priority priority;
	MailFormat mailformat;
	MailFlag mailflag;
	MailPrivateTag mailprivatetag;
	String subject;
	//below is for boolean value, 0 means false, other means true, derby doesn't support bolean value
	int attachments;
	int read;
	
	int folderID;
	int accountID;
	String dataFilePath;
	
	private boolean isNewCreated;

	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(
			this);

	public Mail(String uid, String date, String sender, String receivers,
			String ccers, Priority priority, String subject, int attachments,
			int read, int folderID) {
		this.date = date;
		this.sender = sender;
		this.receivers = receivers;
		this.ccers = ccers;
		this.priority = priority;
		this.subject = subject;
		this.attachments = attachments;
		this.read = read;
		this.folderID = folderID;
	}

	public Mail() {
		this.receivers = "";
		this.sender ="";
		this.ccers ="";
		this.subject="";
		this.date = "";
		this.dataFilePath ="";
		this.bccers="";
		this.priority = Priority.NORMAL;
		this.mailflag = Mail.MailFlag.DRAFT;
		this.mailformat = MailFormat.PLAIN;
		this.mailprivatetag = MailPrivateTag.NONE;
		this.uid = "";

	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(propertyName,
				listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(propertyName,
				listener);
	}

	public String getDate() {
		return date;
	}

	public void setCurrentDate() {
		final SimpleDateFormat format = new  SimpleDateFormat("yyyy-MM-dd HH:mm");  
		Date currentDate = new Date();
		this.date = format.format(currentDate);
	}



	public String getSubject() {
		return subject;
	}

	public void setSubject(String subject) {
		this.subject = subject;
	}

	public int getAttachments() {
		return attachments;
	}

	public void setAttachments(int attachments) {
		this.attachments = attachments;
	}

	public boolean isRead() {
		if(read==0)
			return false;
		return true;
	}

	public void setRead(int read) {
		this.read = read;
	}
	
	public void setReadToDatabase(int read){
		int oldValue = this.read;
		this.read= read;
		firePropertyChange("have_readed", oldValue, read);
	}

	public String getSender() {
		return sender;
	}

	public void setSender(String sender) {
		this.sender = sender;
	}




	public String getReceivers() {
		return receivers;
	}

	public void setReceivers(String receivers) {
		this.receivers = receivers;
	}

	public String getCcers() {
		return ccers;
	}

	public void setCcers(String ccers) {
		this.ccers = ccers;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Priority getPriority() {
		return priority;
	}

	public void setPriority(Priority priority) {
		this.priority = priority;
	}

	public int getFolderID() {
		return folderID;
	}

	public void setFolderID(int folderID) {
		this.folderID = folderID;
	}
	
	public void setFolderIDToDataBase(int folderid) {
		int oldValue = this.folderID;
		this.folderID= folderid;
		firePropertyChange("folderID", oldValue, folderid);
	}
	

/*	public String getAccountName() {
		return AccountName;
	}

	public void setAccountName(String accountName) {
		AccountName = accountName;
	}*/
	
	public boolean hasAttachment(){
		if(attachments==0)
			return false;
		 return true;
	}

	public int getRead() {
		return read;
	}

	public String getDataFilePath() {
		return dataFilePath;
	}

	public void setDataFilePath(String dataFilePath) {
		this.dataFilePath = dataFilePath;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public String getBccers() {
		return bccers;
	}

	public void setBccers(String bccers) {
		this.bccers = bccers;
	}

	public MailFormat getMAILFORMAT() {
		return mailformat;
	}

	public void setMAILFORMAT(MailFormat mailformat) {
		this.mailformat = mailformat;
	}

	public MailFlag getMailflag() {
		return mailflag;
	}

	public void setMailflag(MailFlag mailflag) {
		this.mailflag = mailflag;
	}
	public void setMailflagToDatabase(MailFlag mailflag) {
		
		MailFlag oldValue = this.mailflag;
		this.mailflag= mailflag;
		firePropertyChange("mail_flag", oldValue, mailflag);
	}

	public MailPrivateTag getMailprivatetag() {
		return mailprivatetag;
	}

	public void setMailprivatetag(MailPrivateTag mailprivatetag) {
		this.mailprivatetag = mailprivatetag;
	}
	public void setMailTagToDatabase(MailPrivateTag tag) {
		MailPrivateTag oldValue = this.mailprivatetag;
		this.mailprivatetag= tag;
		firePropertyChange("mail_tag", oldValue, tag);
	}

	public boolean isNewCreated() {
		return isNewCreated;
	}

	public void setNewCreated(boolean isNewCreated) {
		this.isNewCreated = isNewCreated;
	}
	
	protected void firePropertyChange(String propertyName, Object oldValue,
			Object newValue) {
		propertyChangeSupport.firePropertyChange(propertyName, oldValue,
				newValue);
	}

	public String getUid() {
		return uid;
	}

	public void setUid(String uid) {
		this.uid = uid;
	}
	
	public boolean hasCcAddress(){
		if(ccers==null||ccers.equalsIgnoreCase("")||ccers.equalsIgnoreCase(" ")||ccers.equalsIgnoreCase("\r\n"))
	     return false;
		return true;
	}
	
	public void setDateFromOutSide(Date messagedate){
		final SimpleDateFormat format = new  SimpleDateFormat("yyyy-MM-dd HH:mm");  
		this.date = format.format(messagedate);
	}

	public void setDate(String date) {
		this.date = date;
	}

	public int getAccountID() {
		return accountID;
	}

	public void setAccountID(int accountID) {
		this.accountID = accountID;
	}
}
