package org.eclipse.samples.organizer.models;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Date;


public class Task {	
	public enum Priority {LOW, NORMAL, HIGH};
	
	String id;
	Date dueDate;
	Date reminderDate;
	Priority priority = Priority.NORMAL;
	String subject;
	String details;
	boolean complete;

	
	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(
			this);

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(propertyName,
				listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(propertyName,
				listener);
	}

	protected void firePropertyChange(String propertyName, Object oldValue,
			Object newValue) {
		propertyChangeSupport.firePropertyChange(propertyName, oldValue,
				newValue);
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public Date getDueDate() {
		return dueDate;
	}

	public void setDueDate(Date dueDate) {
		Date oldValue = this.dueDate;
		this.dueDate = dueDate;
		firePropertyChange("dueDate", oldValue, dueDate);
	}

	public Date getReminderDate() {
		return reminderDate;
	}

	public void setReminderDate(Date reminderDate) {
		Date oldValue = this.reminderDate;
		this.reminderDate = reminderDate;
		firePropertyChange("reminderDate", oldValue, reminderDate);
	}

	public Priority getPriority() {
		return priority;
	}

	public void setPriority(Priority priority) {
		Priority oldValue = this.priority;
		this.priority = priority;
		firePropertyChange("priority", oldValue, priority);
	}

	public String getSubject() {
		return subject;
	}

	public void setSubject(String subject) {
		String oldValue = this.subject;
		this.subject = subject;
		firePropertyChange("subject", oldValue, subject);
	}

	public String getDetails() {
		return details;
	}

	public void setDetails(String details) {
		String oldValue = this.details;
		this.details = details;
		firePropertyChange("details", oldValue, details);
	}

	public boolean isComplete() {
		return complete;
	}

	public void setComplete(boolean complete) {
		boolean oldValue = this.complete;
		this.complete = complete;
		firePropertyChange("complete", oldValue, complete);
	}
} 
