package org.eclipse.samples.organizer.tasks.format;

import java.io.OutputStream;
import java.io.Reader;
import java.util.Date;

import org.eclipse.samples.organizer.models.Task;

public class RFC2445Format {
	public void saveTask(Task task, OutputStream out) {
		saveTask(task, new RFC2445Writer(out));
	}

	private void saveTask(Task task, RFC2445Writer writer) {
		writer.write("BEGIN", "VTODO");
		writer.write("UID", task.getId());
		writer.write("DUE", task.getDueDate());
		writer.write("SUMMARY", task.getSubject());
		writer.write("STATUS", getStatus(task));
		writer.write("PRIORITY", getPriority(task));
		writer.write("DESCRIPTION", task.getDetails());
		writer.write("END", "VTODO");
	}

	/**
	 * See RFC 2445 section 4.8.1.11.
	 * @param task
	 * @return
	 */
	private String getStatus(Task task) {
		if (task.isComplete()) return "COMPLETED";
		return "NEEDS-ACTION";
	}

	/**
	 * See RFC 2445 section 4.8.1.9.
	 * 
	 * @param task
	 * @return
	 */
	private int getPriority(Task task) {
		if (task.getPriority() == Task.Priority.HIGH) return 1;
		if (task.getPriority() == Task.Priority.NORMAL) return 5;
		if (task.getPriority() == Task.Priority.LOW) return 9;
		return 0;
	}

	public Task loadTask(Reader in) {
		RFC2445Reader reader = new RFC2445Reader(in);
		final Task task = new Task();
		reader.accept(new RFC2445Visitor() {
			public void visit(String key, String value) {
				if ("UID".equalsIgnoreCase(key)) {
					task.setId(value);
				} else if ("DUE".equalsIgnoreCase(key)) {
					task.setDueDate(getDate(value));
				}
			}
		});
		return task;
	}

	Date getDate(String value) {
		// TODO Auto-generated method stub
		return null;
	}
}
