package org.eclipse.samples.organizer.tasks.format;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

public class RFC2445Writer {

	private final PrintWriter out;
	
	private static final DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd'T'HHmmss'z'");

	public RFC2445Writer(OutputStream out) {
		this(new PrintWriter(out));
	}

	public RFC2445Writer(PrintWriter out) {
		this.out = out;
	}

	public void write(String key, String value) {
		if (value == null) return;
		out.print(key);
		out.print(':');
		int column = key.length() + 1;
		int start = 0;
		int end = 73 - column;
		while (start < value.length()) {
			if (end > value.length()) end = value.length();
			if (start > 0) out.print(' ');
			out.write(value, start, end-start);
			out.print("\r\n");
			start = end;
			end = start + 73;
		}
		out.flush();
	}

	public void write(String key, Date date) {
		if (date == null) return;
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.setTimeZone(TimeZone.getTimeZone("Etc/UTC"));
		write(key, dateFormat.format(calendar.getTime()));
	}
	
	public void write(String key, int i) {
		write(key, String.valueOf(i));
	}

}
