/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.dnd.outlook;

import java.io.ByteArrayInputStream;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

/**
 * @author Wayne Beaton
 *
 */
public class ObjectDescriptorTransfer extends ByteArrayTransfer {
	
	private static final String OBJECT_DESCRIPTOR_NAME = "Object Descriptor";
	private static final int OBJECT_DESCRIPTOR_ID = registerType(OBJECT_DESCRIPTOR_NAME);
	
	private static final ObjectDescriptorTransfer instance = new ObjectDescriptorTransfer();

	public static ObjectDescriptorTransfer getInstance () {
		return instance;
	}
	public ObjectDescriptorTransfer() {
		super();
	}

	@Override
	protected int[] getTypeIds() {
		return new int[] {OBJECT_DESCRIPTOR_ID};
	}

	@Override
	protected String[] getTypeNames() {
		return new String[] {OBJECT_DESCRIPTOR_NAME};
	}
	
	@Override
	public Object nativeToJava(TransferData transferData) {
		byte[] data = (byte[])super.nativeToJava(transferData);
		ByteArrayReader in = new ByteArrayReader(data);
	
		// Size of the structure. Should be the same as the length of the byte array.
		in.nextULONG(); 
		
		// I have no idea what the following values are. They all seem to be zero anyway...
		// This structure doesn't seem to match my understanding. Since I don't really need 
		// any of these values at present, I don't really care what they are. At least not yet...
		in.nextULONG(); 
		in.nextULONG(); 
		in.nextULONG();
		in.nextULONG(); 
		in.nextDWORD();
		
		// The name of the application that is the source of the drag event.
		// This is an offset into the byte array of a zero-terminated string.
		long dragSourceNameOffset = in.nextDWORD(); 
		
		// Description of what's being dragged.
		// This is an offset into the byte array of a zero-terminated string.
		long descriptionOffset = in.nextDWORD(); 
		
		// Use the offsets to obtain the values.
		String dragSourceName = extractZeroTerminatedString(data, (int)dragSourceNameOffset);
		String description = extractZeroTerminatedString(data, (int)descriptionOffset);
		
		return new ObjectDescriptor(dragSourceName, description);
	}
	
	private String extractZeroTerminatedString(byte[] data, int srcOfCopyOffset) {
		StringBuilder builder = new StringBuilder();
		int offset = srcOfCopyOffset;
		while (true) {
			int value = data[offset++] + data[offset++] * 256;
			if (value == 0) break;
			builder.append((char)value);
		}
		return builder.toString();
	}

	class ByteArrayReader {
		ByteArrayInputStream stream;
		
		public ByteArrayReader(byte[] bytes) {
			this(new ByteArrayInputStream(bytes));
		}
		
		public ByteArrayReader(ByteArrayInputStream in) {
			stream = in;
		}
		
		public long nextULONG() {
			return nextValue(8);
		}
		
		public long nextDWORD() {
			return nextValue(4);
		}

		private long nextValue(int count) {
			long value = 0;
			long multiplier = 1;
			for(int index=0;index<count;index++) {
				value = value + (stream.read() * multiplier);
				multiplier *= 256;
			}
			
			return value;
		}
	}
}
