/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.dnd.outlook;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.eclipse.samples.organizer.dnd.Activator;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.models.Task.Priority;
import org.eclipse.samples.organizer.ui.task.dnd.DropHandler;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;

public class OutlookDropHandler implements DropHandler {
	private ObjectDescriptorTransfer objectDescriptorTransfer = ObjectDescriptorTransfer.getInstance();
	private TextTransfer textTransfer = TextTransfer.getInstance();

	private Transfer[] transferTypes = new Transfer[] {
			objectDescriptorTransfer
	};
	

	public Integer getWeight() {
		return new Integer(100);
	}
	
	public Transfer[] getSupportedTransfers() {
		return transferTypes;
	}
	
	public List<Task> createTasksFromDropEvent(DropTargetEvent event) {
		/*
		 * Iterate through the data types in the event to see if one of
		 * them is an "Object Descriptor" with a drag source equal to "Outlook"
		 * (i.e. the source of the drag event is Outlook). I iterate through
		 * the data types rather than just look at the currentDataType since
		 * other drop handlers may add transfer types that precede 
		 * "Object Descriptor" and so it may not be the first in the list...
		 */
		for (TransferData transferData : event.dataTypes) {
			if (objectDescriptorTransfer.isSupportedType(transferData)) {
				ObjectDescriptor descriptor = (ObjectDescriptor)objectDescriptorTransfer.nativeToJava(transferData);
				if ("Outlook".equals(descriptor.getDragSource())) 
					return extractTasksFromDropEvent(event);
			}
		}
		return null;
	}
	
	private List<Task> extractTasksFromDropEvent(DropTargetEvent event) {
		for (TransferData transferData : event.dataTypes) {
			if (textTransfer.isSupportedType(transferData)) {
				final String text = (String) textTransfer.nativeToJava(transferData);
				if (text == null)return null;
				return createTasksFromOutlookText(text);
			}
		}
		return null;
	}

	private List<Task> createTasksFromOutlookText(String text) {
		Map<String, String>[] data = TabbedDataExtractor.getData(text);
		List<Task> tasks = new ArrayList<Task>();
		for(int index=0;index<data.length;index++) {
			Map<String, String> entry = data[index];
			Task task = new Task();
			task.setSubject(entry.get("Subject"));
			task.setDueDate(convertToDate(entry.get("Due Date")));
			task.setComplete("Completed".equals(entry.get("Status")));
			// TODO converting the text value is probably inappropriate.
			Task.Priority priority = getPriority(entry.get("Priority"));
			if (priority == null) priority = Task.Priority.NORMAL;
			task.setPriority(priority);
			tasks.add(task);
		}
		return tasks;
	}
	
	private Priority getPriority(String string) {
		// TODO The priority doesn't seem to be getting passed in.
		return Task.Priority.NORMAL;
	}

	private Date convertToDate(String string) {
		if (string == null) return null;
		if ("none".equalsIgnoreCase(string)) return null;
		
		/*
		 * TODO Currently assumes that date is delivered in an "enhanced" short format.
		 * I assume that this is *not* a reasonable assumption in all locales.
		 */
		String[] parts = string.split(" ");
		if (parts.length == 2) {
			DateFormat format = DateFormat.getDateInstance(DateFormat.SHORT);
			try {
				return format.parse(parts[1]);
			} catch (ParseException e) {
				Activator.logError(e);
			}
		}
		
		return null;
	}
}
