/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*******************************************************************************/
package org.eclipse.samples.organizer.ui.contact.helper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.samples.organizer.contacts.filters.ContactFilter;
import org.eclipse.samples.organizer.ui.model.FeatureHelper;
import org.eclipse.samples.organizer.ui.contact.Activator;
import org.eclipse.samples.organizer.ui.contact.dnd.ContactDropTarget;
import org.eclipse.samples.organizer.ui.contact.wizards.ImportContactsWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.forms.HyperlinkGroup;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;

public class ContactViewFeatureHelper implements FeatureHelper {

	private FormToolkit toolkit;
//	private Button filterComplete;

	public void createSwitcherContentArea(final Composite parent) {
		parent.setLayout(new FillLayout());
		Composite content = new Composite(parent, SWT.NONE);
		content.setLayout(new GridLayout(1, true));
		content.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		
		toolkit = new FormToolkit(parent.getDisplay());
		toolkit.getHyperlinkGroup().setHyperlinkUnderlineMode(HyperlinkGroup.UNDERLINE_HOVER);
		
		Hyperlink createContact = toolkit.createHyperlink(content, "Create contact...", SWT.NONE);
		createContact.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		createContact.addHyperlinkListener(new HyperlinkAdapter() {
			public void linkActivated(HyperlinkEvent e) {
				//new AddContactAction("").run();
				createContact(parent.getShell());
			}			
		});
		
		Hyperlink refreshTask = toolkit.createHyperlink(content, "Refresh list", SWT.NONE);
		refreshTask.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
			
		Label separator = toolkit.createSeparator(content, SWT.HORIZONTAL);
		separator.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		/*filterComplete = toolkit.createButton(content, "Hide complete contacts", SWT.CHECK);
		filterComplete.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		filterComplete.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ContactFilter.instance.setHideCompleteTasks(filterComplete.getSelection());
			}
		});*/
		final PropertyChangeListener filterCompleteChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				//updateFilterComplete();
			}
		};
		ContactFilter.instance.addPropertyChangeListener(ContactFilter.NAME_START, filterCompleteChangeListener);

	/*	filterComplete.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				ContactFilter.instance.removePropertyChangeListener(ContactFilter.HIDE_COMPLETE, filterCompleteChangeListener);
			}			
		});*/
		
		//updateFilterComplete();
		
		
		content.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				toolkit.dispose();
			}			
		});
		
		try {
			Activator.getDefault().getWorkbench().showPerspective("org.ecipse.samples.organizer.contact.ui.perspective", Activator.getDefault().getWorkbench().getActiveWorkbenchWindow());
		} catch (WorkbenchException e) {
			e.printStackTrace();
		}
	}

/*	protected void updateFilterComplete() {
		filterComplete.setSelection(ContactFilter.instance.isHideCompleteTasks());		
	}*/

	private void createContact(final Shell shell) {
		final ArrayList<Contact> contacts = new ArrayList<Contact>();
		Contact newContact = new Contact();
		contacts.add(newContact);
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ImportContactsWizard wizard = new ImportContactsWizard(contacts);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				dialog.open();
			}
		});
	}

	public void configureSwitcherButton(Button button) {
		new ContactDropTarget(button);
	}

	public String getSummary() {
		return String.valueOf(getCount()) + " contacts";
	}

	private int getCount() {
		try {
			return OrganizerManager.instance.count(ContactFilter.instance);
		} catch (BackendException e) {
			return 0;
		}
	}
}
