/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.contact.wizards;

import java.util.List;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

public class ImportContactMainWizardPage extends WizardPage {
	
	private List<Backend> backends;
	private Backend selectedBackend;
	
	public ImportContactMainWizardPage(List<Backend> backends, Backend defaultBackend) {
		super("Location");
		
		assert !backends.isEmpty();
		assert backends.contains(defaultBackend);
		
		setTitle("Location for this task");
		setDescription("Where should this task be stored?");
		
		this.backends = backends;
		this.selectedBackend = defaultBackend;
	}

	public void createControl(Composite parent) {
		Composite composite =  new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(2, false));

		createLocationLabel(composite);		
		createLocationCombobox(composite);	
		
		setControl(composite);		
	}

	private void createLocationLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Location:");
	}

	private void createLocationCombobox(Composite composite) {
		final Combo location = new Combo(composite, SWT.READ_ONLY);
		location.setItems(getBackendNames());
		location.setText(getSelectedBackendName()); 
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		location.setLayoutData(gridData);
		location.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setSelectedBackend(location.getText());
				getContainer().updateButtons();
			}
		});
	}

	private String getSelectedBackendName() {
		if (selectedBackend == null) return "";
		return selectedBackend.getName();
	}

	public String[] getBackendNames() {
		List<Backend> backends = OrganizerManager.instance.getBackends();
		String[] names = new String[backends.size()];
		for (int index=0;index<backends.size();index++) {
			names[index] = backends.get(index).getName();			
		}
		return names;
	}

	private void setSelectedBackend(String text) {
		for (Backend backend : backends) {
			if (backend.getName().equals(text)) {
				setSelectedBackend(backend);
				return;
			}
		}
		setSelectedBackend((Backend)null);
	};
	
	private void setSelectedBackend(Backend backend) {
		selectedBackend = backend;
		// TODO It would be better to use an observer here.
		getImportContactsWizard().setBackend(backend);
	}

	private ImportContactsWizard getImportContactsWizard() {
		return (ImportContactsWizard)getWizard();
	}

	public IWizardPage getNextPage() {
		return getWizard().getNextPage(this);
	}

	public boolean canFlipToNextPage() {
		if (!hasBackend()) return false;
		return getNextPage() != null;
	}

	public boolean hasBackend() {
		return selectedBackend != null;
	}
}

