/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*******************************************************************************/
package org.eclipse.samples.organizer.ui.contact.wizards;


import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

public class ImportContactWizardPage extends WizardPage {

	private ContactNameFormat contactName;
	private final Contact contact;
	private Text firstNameField;
	private Combo fullNameCombo;
	private Text lastNameField;
	private Text middleNameText;
	private Text primaryEmail;
	private List mailsListField;
	private Text emailText;
	
	
	public ImportContactWizardPage(Contact contact) {
		super("Add a new Contact information");
		this.contact = contact;
		setTitle("Add a new Contact information");
		setDescription("Provide information about this contact");	
	}

	class ContactNameFormat{
		private String firstName;
		private String middleName;
		private String  lastName;
		private String fullName;
		int format;
		public ContactNameFormat(){
			firstName = "";
			middleName = "";
			lastName = "";
			fullName = "";
		}
		public String getFirstName() {
			return firstName;
		}
		public void setFirstName(String firstName) {
			this.firstName = firstName;
			
		}
		public String getMiddleName() {
			return middleName;
		}
		public void setMiddleName(String middleName) {
			this.middleName = middleName;
		}
		public String getLastName() {
			return lastName;
		}
		public void setLastName(String lastName) {
			this.lastName = lastName;
		}
		public String getFullName() {
			return fullName;
		}
		public void setFullName(String fullName) {
			this.fullName = fullName;
		}
		
		public String produceNormalName(){
			return lastName + " " + firstName;
		}
		
		private String produceCommaName(){
			return new StringBuilder(firstName).append(",").append(lastName).toString();
		}
		
		public String[] getAllNameSelection(){
			String[] nameString = new String[2];
			nameString[0] = produceNormalName();
			nameString[1] = produceCommaName();
			return nameString;
		}
		public String getDefaultFullNameByFormat(int format){
			if(format<0||format >getAllNameSelection().length)
				return "";
			return getAllNameSelection()[format];
		}
	}
	
	
	public void createControl(Composite parent) {
		Composite composite =  new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(9, false));

		createFirstNameLabel(composite);		
		createFirstNameText(composite);	
		createMiddleNameLabel(composite);	
		createMiddleNameText(composite);	
		createLastNameLabel(composite);		
		createLastNameText(composite);	
		createFullNameLabel(composite);		
	    createFullNameCombo(composite);	
		createEmailLabel(composite);		
	    createEmailText(composite);	
	    createEmailAddButton(composite);
	    createMailsListField(composite);
	    createPrimaryLabel(composite);		
	    createPrimaryText(composite);	
	    
		/*createCompleteCheckbox(composite);	
		createReminderDateLabel(composite);		
		createReminderDatePicker(composite);	
		createFiller(composite);
		createSubjectLabel(composite);		
		createSubjectField(composite);
		createDetailsLabel(composite);		
		createDetailsField(composite);*/
		
		setControl(composite);		
		contactName = new ContactNameFormat();
	}

	private void createFirstNameLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("First Name:");
	}
	private void createLastNameLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Last Name:");
	}
	private void createFullNameLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Full Name:");
	}
	private void createPrimaryLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Primary Email:");
	}
	private void createEmailLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Email:");
	}
	private void createMiddleNameLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Middle Name:");
	}
	private void createFiller(Composite composite) {
		new Label(composite, SWT.NONE);
	}
	
	private void createFirstNameText(Composite composite) {
		firstNameField = new Text(composite, SWT.BORDER);
		GridData gridData = new GridData();
		//gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		firstNameField.setLayoutData(gridData);
		firstNameField.setText(contact.getFirstName());
		firstNameField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				contactName.setFirstName(firstNameField.getText());
				fullNameCombo.setItems(contactName.getAllNameSelection());
				fullNameCombo.setText(contactName.getDefaultFullNameByFormat(0));
				fullNameCombo.update();
				contact.setFullName(fullNameCombo.getText());
				
			}
		});
		addDropListener(firstNameField);		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(firstNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
		
	}
	private void createLastNameText(Composite composite) {
		lastNameField = new Text(composite, SWT.BORDER);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		//gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		lastNameField.setLayoutData(gridData);
		lastNameField.setText(contact.getLastName());
		
		lastNameField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				contactName.setLastName(lastNameField.getText());
				fullNameCombo.setItems(contactName.getAllNameSelection());
				fullNameCombo.setText(contactName.getDefaultFullNameByFormat(0));
				fullNameCombo.update();
				contact.setFullName(fullNameCombo.getText());
				//System.out.println(contact.getFullName());
			}
		});
		addDropListener(lastNameField);		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(lastNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
	}
	private void createFullNameCombo(Composite composite) {
		fullNameCombo = new Combo(composite, SWT.READ_ONLY);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 8;
		fullNameCombo.setLayoutData(gridData);
		//fullNameCombo.setItems(this.contactName.getAllNameSelection());
		fullNameCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
			}
		});
	//	addDropListener(firstNameField);		
	//	PlatformUI.getWorkbench().getHelpSystem().setHelp(firstNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
	}
	
	private void createPrimaryText(Composite composite) {
		primaryEmail = new Text(composite, SWT.READ_ONLY);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 8;
		primaryEmail.setLayoutData(gridData);
		
	//	addDropListener(firstNameField);		
	//	PlatformUI.getWorkbench().getHelpSystem().setHelp(firstNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
	}
		private void createEmailText(Composite composite) {
			emailText = new Text(composite, SWT.BORDER);
			GridData gridData = new GridData();
			gridData.horizontalAlignment = SWT.FILL;
			gridData.grabExcessHorizontalSpace = true;
			gridData.horizontalSpan = 6;
			emailText.setLayoutData(gridData);
			emailText.setText("");
			emailText.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					//contact.setFirstName(firstNameField.getText());
				//	setTitle(firstNameField.getText());
				}
			});
		//	addDropListener(firstNameField);		
		//	PlatformUI.getWorkbench().getHelpSystem().setHelp(firstNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
		}
		
		
		private void createEmailAddButton(Composite composite) {
			final Button addEmailButton = new Button(composite, SWT.NONE);
			GridData gridData = new GridData();
			gridData.horizontalAlignment = SWT.FILL;
			gridData.grabExcessHorizontalSpace = true;
			gridData.horizontalSpan = 1;
			addEmailButton.setLayoutData(gridData);
			addEmailButton.setText("Add this Email");
			addEmailButton.addSelectionListener(new SelectionAdapter(){
				public void widgetSelected(SelectionEvent e) {
					if(emailText.getText().equalsIgnoreCase("")){
						return;
					}
					String emailAddress = emailText.getText();
					if(!emailAddress.contains("@")){
						setTitle("Please input legal email address like this: example@good.com");
						return;
					}
						
					if(mailsListField.getItemCount()==0){
						primaryEmail.setText(emailAddress);
						contact.setPrimaryMailAddress(primaryEmail.getText().trim());
						primaryEmail.update();
						contact.setPrimaryMailAddress(emailAddress);
					}
					mailsListField.add(emailAddress);
					mailsListField.update();
					emailText.setText("");
				}
			});
		//	addDropListener(firstNameField);		
		//	PlatformUI.getWorkbench().getHelpSystem().setHelp(firstNameField, "org.eclipse.samples.organizer.ui.contact.firstNameField");
		}
	/*private void createReminderDatePicker(Composite composite) {
		final DatePickerCombo reminderDatePicker = new DatePickerCombo(composite, SWT.BORDER);
		reminderDatePicker.setDateFormat(DateFormat.LONG);
		reminderDatePicker.setDate(contact.getReminderDate());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 1;
		reminderDatePicker.setLayoutData(gridData);
		reminderDatePicker.addDateSelectionListener(new DateSelectionListener() {
			public void dateSelected(DateSelectedEvent event) {
				contact.setReminderDate(event.date);
			}
		});
		PlatformUI.getWorkbench().getHelpSystem().setHelp(reminderDatePicker, "org.eclipse.samples.organizer.ui.contact.reminderDateField");
	}*/

	/*private void createCompleteCheckbox(Composite composite) {
		final Button completeButton = new Button(composite, SWT.CHECK);
		completeButton.setText("Complete");
		completeButton.setSelection(contact.isComplete());
		completeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				contact.setComplete(completeButton.getSelection());
			};
		});
	}*/


	private void createMiddleNameText(Composite composite) {
		middleNameText = new Text(composite, SWT.BORDER);
		GridData gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		middleNameText.setLayoutData(gridData);
		middleNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
			//	contact.set
			}
		});
		//addDropListener(subjectField);		
		//PlatformUI.getWorkbench().getHelpSystem().setHelp(subjectField, "org.eclipse.samples.organizer.ui.contact.subjectField");
	}

	private void createDetailsLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Details:");
		GridData gridData = new GridData();
		gridData.horizontalSpan = 3;
		label.setLayoutData(gridData);
	}

	private void createMailsListField(Composite composite) {
		mailsListField = new List(composite, SWT.MULTI | SWT.SINGLE | SWT.V_SCROLL | SWT.BORDER);
		//mailsField.setText(contact.getDetails() == null ? "" : contact.getDetails());
		//mailsListField.setItems(new String[]{"hali"});
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 9;
		//gridData.widthHint = 300;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		//gridData.heightHint = 300;
		mailsListField.setLayoutData(gridData);
		/*mailsField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				//contact.setDetails(detailsField.getText());
			}
		});*/
		//addDropListener(detailsField);
		//PlatformUI.getWorkbench().getHelpSystem().setHelp(detailsField, "org.eclipse.samples.organizer.ui.contact.detailField");
	}
	
	private void addDropListener(final Text widget) {
		DropTarget dropTarget = new DropTarget(widget, DND.DROP_COPY | DND.DROP_DEFAULT);
		dropTarget.setTransfer(new Transfer[] {TextTransfer.getInstance()});
		dropTarget.addDropListener(new DropTargetAdapter() {
			TextTransfer textTransfer = TextTransfer.getInstance();

			public void dragEnter(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOperationChanged(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOver(DropTargetEvent event) {
			}

			public void drop(DropTargetEvent event) {

				if (textTransfer.isSupportedType(event.currentDataType)) {
					String text = (String) textTransfer.nativeToJava(event.dataTypes[1]);
					if (text != null) {
						widget.setText(text);
					}
				}
			}
		});
	}
	
	public IWizardPage getNextPage() {
		return getWizard().getNextPage(this);
	}

	public boolean canFlipToNextPage() {
		return getNextPage() != null;
	}
}

