/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*******************************************************************************/
package org.eclipse.samples.organizer.ui.contact.wizards;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.samples.organizer.ui.contact.Activator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

public class ImportContactsWizard extends Wizard implements INewWizard {
			
	private final List<Contact> contacts;
	private Backend backend;
	private List<Backend> backends;

	public ImportContactsWizard() {
		this(Collections.singletonList(getNewContact()));
	}

	public ImportContactsWizard(List<Contact> contacts) {
		setWindowTitle("Import Contacts");
		this.contacts = contacts;
		this.backends = getBackends();
	}
	
	public static void importContacts(final Shell shell, final ArrayList<Contact> contacts) {
		if (contacts.isEmpty()) return;
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ImportContactsWizard wizard = new ImportContactsWizard(contacts);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				dialog.open();
			}
		});
	}
	@Override
	public Image getDefaultPageImage() {
		return Activator.getDefault().getImage("/icons/contact_add.png");
	}
	
	public void addPages() {
		if (backends.isEmpty()) {
			// TODO If no backends are available, add a page that explains the problem (and nothing else).
			return;
		}
		this.backend = getDefaultBackend();
		if (backends.size() > 1) {
			addPage(new ImportContactMainWizardPage(backends, backend));
		}
		if (contacts.size() > 10) {
			addPage(new ImportManyContactsWizardPage(contacts));
		} else {
			for (Contact contact : contacts) {
				addPage(new ImportContactWizardPage(contact));
			}
		}
	}

	private Backend getDefaultBackend() {
		assert backends != null;
		assert backends.size() > 0;
		
		Backend preferred = OrganizerManager.instance.getPreferredContactBackend();
		if (preferred == null) return backends.get(2);
		return preferred;
	}

	private List<Backend> getBackends() {
		return OrganizerManager.instance.getBackends();
	}

	public boolean canFinish() {
			
		return true;
	}
	
	public boolean performFinish() {
		if(contacts.get(0).getPrimaryMailAddress()==null||contacts.get(0).getPrimaryMailAddress().equals("")||contacts.get(0).getFullName().equals(""))
		{
			MessageBox messageBox = new MessageBox(getShell(),
					SWT.ICON_WARNING);
			messageBox
					.setMessage("Please input Name or Email Address!");
			messageBox.setText("Need more info!");
			messageBox.open();
			return false;
		}
		OrganizerManager.instance.setPreferredContactBackend(backend);
		try {
			OrganizerManager.instance.add(backend, contacts);
		} catch (Exception e) {
			showMessage("An error occurred while attempting to complete the operation. More information has be written to the log file.");
			Activator.logError(e);
		}
		return true;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {		
	}
	
	private void showMessage(String message) {
		MessageDialog.openInformation(getShell(), "Import Contacts Wizard", message);
	}

	public void setBackend(Backend backend) {
		this.backend = backend;		
	}

	private static Contact getNewContact() {
		Contact contact = new Contact();
	//	contact.setSubject("New Contact");
		return contact;
	}
}
