/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.eclipse.samples.organizer.ui.mail";

	// The shared instance
	private static Activator plugin;
	
	public static final String DEFAULT_ACCOUNT_FILE = "accounts.xml";

	
	public static final String IMAGE_LOW = "/icons/low.gif";
	public static final String IMAGE_NORMAL = "/icons/normal.gif";
	public static final String IMAGE_HIGH = "/icons/high.gif";
	public static final String IMAGE_DRAFT = "/icons/read.gif";
	public static final String IMAGE_NEW = "/icons/draftview.gif";
	public static final String IMAGE_REPLYSYMBOL = "/icons/replysymbol.gif";
	public static final String IMAGE_MAIL_OUT = "/icons/mail_out.png";
	public static final String IMAGE_MAIL_SAVE = "/icons/saveas.png";
	public static final String IMAGE_MAIL_LOWPRI = "/icons/low_pri.gif";
	public static final String IMAGE_MAIL_HIGHPRI = "/icons/high_pri.gif";
	public static final String IMAGE_MAIL_NORMALPRI = "/icons/normal_pri.png";
	public static final String IMAGE_MAIL_REPLY = "/icons/reply.gif";
	public static final String IMAGE_MAIL_BIGREPLY = "/icons/big_reply.gif";
	public static final String IMAGE_MAIL_REPLYALL = "/icons/replyall.gif";
	public static final String IMAGE_MAIL_BIGREPLYALL = "/icons/big_replyall.gif";
	public static final String IMAGE_MAIL_FORWARD = "/icons/forward.gif";
	public static final String IMAGE_MAIL_BIGFORWARD = "/icons/big_forward.gif";
	public static final String IMAGE_MAIL_NEXT = "/icons/next.gif";
	public static final String IMAGE_MAIL_PREVIOUS = "/icons/previous.gif";
	public static final String IMAGE_MAIL_DELETE = "/icons/delete.gif";
	public static final String IMAGE_MAIL_CONTACT = "/icons/addcontact.gif";
	public static final String IMAGE_MAIL_FORMAT = "/icons/format.gif";
	public static final String IMAGE_MAIL_PRINT = "/icons/print.gif";
	public static final String IMAGE_MAIL_FIND = "/icons/find.gif";
	public static final String IMAGE_MAIL_SEND = "/icons/sendmail.gif";
	public static final String IMAGE_MAIL_OPEN = "/icons/mail16_open.png";
	public static final String IMAGE_MAIL16 = "/icons/mail16.png";
	public static final String IMAGE_MAIL_RETRIEVE = "/icons/receive.gif";
	public static final String IMAGE_MAIL_REFRESH = "/icons/refresh.gif";
	public static final String IMAGE_MAIL_ATTACHMENT = "/icons/attachment.gif";
	public static final String IMAGE_MAIL_BIGATTACHMENT = "/icons/bigattachment.gif";
	public static final String IMAGE_MAIL_DRAFT = "/icons/draftview.gif";
	public static final String ACCOUNT_IMAGE_PATH = "/icons/account.gif";
	public static final String TRASH_IMAGE_PATH = "/icons/trash16.png";
	public static final String SENT_IMAGE_PATH = "/icons/mail_out.png";
	public static final String DRAFT_IMAGE_PATH = "/icons/draft.gif";
	public static final String INBOX_IMAGE_PATH = "/icons/inboxfolder.PNG";
	public static final String OUTBOX_IMAGE_PATH = "/icons/outboxfolder.gif";
	public static final String UP_IMAGE_PATH = "/icons/up.gif";
	public static final String DOWN_IMAGE_PATH = "/icons/down.gif";
	public static final String ADDRESSBOOK_IMAGE_PATH = "/icons/addresses.gif";
	public static final String ACTIVITY_IMAGE_PATH = "/icons/activity.gif";
	
	/**";reply.gifaddresses.gif
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}


	public Image getImage(String path) {
		Image image = getImageRegistry().get(path);
		if (image == null) {
			ImageDescriptor descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(getBundle().getSymbolicName(), path);
			getImageRegistry().put(path, descriptor);
			image = getImageRegistry().get(path);
		}
		return image;
	}
	public static void logError(Exception e) {
		logError(new Status(IStatus.ERROR, getSymbolicName(), 0, e.getMessage(), e));
	}

	public static void logError(IStatus status) {
		getDefault().getLog().log(status);
	}

	public static void logError(String message) {
		logError(new Status(IStatus.ERROR, getSymbolicName(), 0, message, null));
	}
	
	public static String getSymbolicName() {
		return getDefault().getBundle().getSymbolicName();
	}
	
	public void showErrorMessageOnStatusbar(String info){
		
			  IWorkbenchWindow workbenchWindow = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			   IActionBars bars = ((WorkbenchWindow)workbenchWindow).getActionBars();
			   IStatusLineManager lineManager = bars.getStatusLineManager();
			   lineManager.setErrorMessage(info);
			
	}
	public void showMessageOnStatusbar(String info){
		
		  IWorkbenchWindow workbenchWindow = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		   IActionBars bars = ((WorkbenchWindow)workbenchWindow).getActionBars();
		   IStatusLineManager lineManager = bars.getStatusLineManager();
		   lineManager.setMessage(info);
}


}
