/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.models.Contact;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.samples.organizer.ui.mail.*;

public class AddContactAction extends Action {
    String firstName;
    String middleName;
    String lastName;
    String emailAddress;
    String fullName;
    
	private Backend backend;
	private List<Backend> backends;
	
	private AddContactAction(){
		firstName = "";
		middleName = "";
		lastName = "";
		emailAddress = "";
		fullName = "";
		this.backends = getBackends();
	}
	public AddContactAction(String address){
		this();
		separteNames(address);
	}
	private List<Backend> getBackends() {
		return OrganizerManager.instance.getBackends();
	}
	private void separteNames(String address) {
		Map<String,String> nameMap= new HashMap<String,String>();
		if(address.trim().equalsIgnoreCase("")){
			return;
		}else if(address.contains("<")&&address.contains(">")){
			String nameInfo = address.split("<")[0];
			emailAddress = address.split("<")[1].split(">")[0].trim();
			int nameDivision = nameInfo.split(" ").length;
			if(nameDivision==1){
				lastName = nameInfo;
				fullName = lastName;
			}else if(nameDivision==2){
				lastName = nameInfo.split(" ")[0];
				firstName =  nameInfo.split(" ")[1];
				fullName = new StringBuilder(lastName).append(" ").append(firstName).toString();
			}else if(nameDivision==3){
				lastName = nameInfo.split(" ")[0];
				middleName = nameInfo.split(" ")[1];
				firstName = nameInfo.split(" ")[2];
				fullName = new StringBuilder(lastName).append(" ").append(middleName).append(" ").append(firstName).toString();
			}
		}
		
	}
	public void run(){
		final ArrayList<Contact> contacts = new ArrayList<Contact>();
		Contact newContact = new Contact();
		newContact.setPrimaryMailAddress(emailAddress);
		newContact.setFirstName(firstName);
		newContact.setMiddleName(middleName);
		newContact.setLastName(lastName);
		newContact.setFullName(fullName);
		contacts.add(newContact);
		final Shell shell = Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				backend = getDefaultBackend();
				OrganizerManager.instance.setPreferredContactBackend(backend);
				try {
					OrganizerManager.instance.add(backend, contacts);
				} catch (Exception e) {
					e.printStackTrace();
					Activator.logError(e);
				}
			}
		});
		
	}
	
	private Backend getDefaultBackend() {
		assert backends != null;
		assert backends.size() > 0;
		
		Backend preferred = OrganizerManager.instance.getPreferredContactBackend();
		if (preferred == null) return backends.get(2);
		return preferred;
	}
	
}
