/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.editors;

import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.samples.organizer.ui.mail.actions.AddContactAction;

public class AddContactEditor {

	
	private Display display;
	private Shell s;
	private Composite composite;
	private Text selectedText;
    private String[] allAddresses;
	private AddContactEditor(){
		
	}
	public AddContactEditor(String addresses){
		allAddresses = new String[addresses.split(",").length];
		allAddresses = addresses.split(",");
		display = Display.getDefault();
		createShell();
		createContentComposite();
		
	
	}

	private void createContentComposite() {
		composite = new Composite(s, SWT.MULTI);
		GridData spec = new GridData();
		spec.horizontalAlignment = GridData.FILL;
		spec.grabExcessHorizontalSpace = true;
		spec.verticalAlignment = GridData.FILL;
		spec.grabExcessVerticalSpace = true;
		composite.setLayoutData(spec);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.horizontalSpacing = 3;
		composite.setLayout(gridLayout);
		createContent(composite);
	}

	private void createContent(Composite cst) {
		createFromLabel(cst);
		createAddressList(cst);
		createSelectedLabel(cst);
		createSelectedText(cst);
		createAddbutton(cst);
		createClosebutton(cst);
	}
	private void createClosebutton(Composite cst) {
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.horizontalSpan = 1;
		final Button closebutton = new Button(cst, SWT.NONE);
		closebutton.setLayoutData(gridData1);
		closebutton.setText("Close");
		closebutton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				s.dispose();
			}
		});
		
	}
	private void createSelectedLabel(Composite cst) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 3;
		gridData.grabExcessHorizontalSpace = true;

		final Label selectedlabel = new Label(cst, SWT.NONE);
		selectedlabel.setLayoutData(gridData);
		selectedlabel.setText("You have selected:");
		selectedlabel.setLayoutData(gridData);
		
	}
	private void createAddbutton(Composite cst) {
		GridData gridData = new GridData();
		gridData.horizontalSpan = 1;
		gridData.grabExcessHorizontalSpace = true;

		final Button b = new Button(cst, SWT.NONE);
		b.setLayoutData(gridData);
		b.setVisible(false);
		
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.horizontalSpan = 1;
		final Button addbutton = new Button(cst, SWT.NONE);
		addbutton.setLayoutData(gridData1);
		addbutton.setText(" Add ");
		addbutton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				   new AddContactAction(selectedText.getText()).run();
				   s.dispose();
			}
		});
		
	}
	private void createSelectedText(Composite cst) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 3;
		gridData.grabExcessHorizontalSpace = true;

	   selectedText = new Text(cst, SWT.READ_ONLY);
	   selectedText.setLayoutData(gridData);
	   selectedText.setLayoutData(gridData);
		
	}
	
	private void createAddressList(Composite cst) {
		
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 3;
		//gridData.widthHint = 300;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		final List addressList = new List(cst,SWT.MULTI | SWT.SINGLE | SWT.V_SCROLL | SWT.BORDER);
		addressList.setItems(allAddresses);
		addressList.setLayoutData(gridData);
		addressList.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				  String selected[] = addressList.getSelection( );
				  selectedText.setText(selected[0]);
			}
		});
	}
	
	private void createFromLabel(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 3;
		gridData.grabExcessHorizontalSpace = true;

		final Label titleLabel = new Label(parent, SWT.NONE);
		titleLabel.setLayoutData(gridData);
		titleLabel.setText("Please select the address you want to add:");
	}
	private void createShell() {
		s = new Shell(display, SWT.CLOSE);
		s.setImage(Activator.getDefault().getImage(Activator.IMAGE_MAIL_CONTACT));
		s.setText("Add Email adress into Contact");
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		s.setLayout(layout);
		s.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				s.dispose();
			}
		});
	}
	public void show(){
		s.setBounds(350, 250, 500, 500);
		s.open();
		while (!s.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}
}
