/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.editors;


import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.models.Mail.Priority;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.jobs.SaveMessageJob;
import org.eclipse.samples.organizer.ui.mail.jobs.SendMessageJob;
import org.eclipse.samples.organizer.ui.mail.main.ExtendedMessage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.*;

import de.spiritlink.richhtml4eclipse.example.views.HTMLView;

public class MailEditor {
	private Display display;
	private Shell s;
	private MenuItem aboutMenuItem;
	private MenuItem copyMenuItem;
	private Menu m;
	private MenuItem sendMenuItem;
	private Menu filemenu;
	private MenuItem exitMenuItem;
	private MenuItem edit;
	private Menu editmenu;
	private MenuItem cutMenuItem;
	private ToolBar bar;
	private Image attachIcon;
	private Image saveIcon;
	private Image sendIcon;
	private Text contentText;
	private Image mailheaderIcon;
	private Image accountIcon;
	private Image formatIcon;
	private Image addressbookIcon;
	private MenuItem help;
	private Menu helpmenu;
	private MenuItem pasteMenuItem;
	private MenuItem separator;
	private MenuItem saveMailMenuItem;
	private ToolItem accountToolItem;
	private ToolItem mailheadToolItem;
	private ToolItem mailformatToolItem;
	private ToolItem sep1;
	private ToolItem sendToolItem;
	private ToolItem attachmentToolItem;
	private ToolItem saveToolItem;
	private MenuItem file;
	private Button ccButton = null;
	private Button ToButton = null;
	private Label priorityLabel = null;
	
	private Button priorityImageButton = null;
	private Text ccText = null;

	private Text ToText = null;
	private Label subjectLabel = null;
	private Text subjectText = null;
	private Text detailsText = null;
	private Label formatLabel = null;
	private String mailToAddress;
	private String mailCcAddress;
	private String mailContent;
	private String mailAccountName;
	private String mailbccAddress;
	private Combo formateCombo = null;
	private Button attachButton;
	private List attachList;
	private Label accountLabel;



	private Mail mail;
	private ExtendedMessage message;
	private Composite composite;
	private Composite bccComponent;
	private Button bccButton;
	private Text bccText;
	private boolean isBccHide;
	private HTMLView html;
	private Composite mailContentAread;
	private Composite mailContentArea;

	public MailEditor(Mail _mail, ExtendedMessage _message) {
		this.mail = _mail;
		this.message = _message;
		display = Display.getDefault();
		initework();
		createShell();

		createMenuBar();
		createToolBar();
		createContentComposite();

	}

	private void initework() {

		// create images for toolbar buttons
		sendIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_SEND);
		saveIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_SAVE);
		attachIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_BIGATTACHMENT);
		mailheaderIcon = Activator.getDefault().getImage(Activator.IMAGE_NEW);
		accountIcon = Activator.getDefault().getImage(Activator.ACCOUNT_IMAGE_PATH);
		formatIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_FORMAT);
		addressbookIcon = Activator.getDefault().getImage(Activator.ADDRESSBOOK_IMAGE_PATH);
		mailAccountName = MailManager.getInstance().getAccountFromID(mail.getAccountID()).getAccountName();
	}

	private void createContentComposite() {
		composite = new Composite(s, SWT.MULTI);
		GridData spec = new GridData();
		spec.horizontalAlignment = GridData.FILL;
		spec.grabExcessHorizontalSpace = true;
		spec.verticalAlignment = GridData.FILL;
		spec.grabExcessVerticalSpace = true;
		composite.setLayoutData(spec);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.horizontalSpacing = 5;
		composite.setLayout(gridLayout);
		createContent(composite);

	}

	private void createContent(Composite parent) {
		createToButton(parent);
		createToText(parent);

		createCCLabel(parent);
		createCCText(parent);

		if(isBccHide){
		createBccButton(parent);
		createBccText(parent);
		}	

		createSubjectLabel(parent);
		createSubjectText(parent);

	/*	createFormatLabel(parent);
		createFormatCombo(parent);*/

		createAttachButton(parent);
		createAttachList(parent);

		createPriorityLabel(parent);
		createPriorityImageButton(parent);

		
		createMailContentArea(parent);
	
	}

	

	private void createMailContentArea(Composite parent) {
		mailContentArea = new Composite(parent,SWT.NONE);
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.horizontalSpacing = 5;
		mailContentArea.setLayout(gridLayout);
		mailContentArea.setLayoutData(gridData);
		if (mail.getMAILFORMAT() == Mail.MailFormat.PLAIN) {
			message.setMessageFormat(Mail.MailFormat.PLAIN);
			createCotentsText(mailContentArea);
		}
		else {
			message.setMessageFormat(Mail.MailFormat.HTML);
			createHTMLEditor(mailContentArea);
		}
	}

	private void createHTMLEditor(Composite parent) {
		mail.setMAILFORMAT(Mail.MailFormat.HTML);
		String htmlString;
		html = new HTMLView(parent,htmlString =mail.isNewCreated()?"": message.getPlainTextFromBodyPart());
		
	}
	private void createHTMLEditorFromPlain(Composite parent,String plaincontent) {
		message.setMessageFormat(Mail.MailFormat.HTML);
		html = new HTMLView(parent,plaincontent);
	}
	
	private void createBccHideComponent(Composite parent) {
		bccComponent = new Composite(parent,SWT.BORDER);
		bccComponent.setLayoutData(new RowData(0,0));
		createBccButton(bccComponent);
		createBccText(bccComponent);
	}

	private void createBccText(Composite parent) {
			   GridData gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;

		bccText = new Text(parent, SWT.BORDER);
		bccText.setLayoutData(gridData);
		bccText.setText(mail.getCcers());
		mailCcAddress = mail.getCcers();
		bccText
				.setToolTipText("Please input legal email address like abc@example.com, use \",\" or \";\" to separate two address");

		bccText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mailbccAddress = bccText.getText();

			}
		});
	
		
	}

	private void createBccButton(Composite parent) {
		bccButton = new Button(parent, SWT.NONE);
		bccButton.setText("Bcc:");
		bccButton.setSize(0, 0);
	}

	private void createAttachButton(Composite parent) {
		attachButton = new Button(parent, SWT.NONE);
		attachButton.setImage(Activator.getDefault().getImage(Activator.IMAGE_MAIL_BIGATTACHMENT));
		if(attachButton.getImage()==null){
			attachButton.setText("Attachment");
		}
		attachButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {

			}

			public void widgetSelected(SelectionEvent e) {
				FileDialog fd = new FileDialog(s, SWT.OPEN);
				fd.setText("Open");
				String selected = fd.open();
				attachList.add(selected);
				message.addAttachments(selected);
			}

		});
	}

	private void createSubjectLabel(Composite parent) {
		subjectLabel = new Label(parent, SWT.NONE);
		subjectLabel.setText("Subject:");
	}

	private void createPriorityLabel(Composite parent) {
		priorityLabel = new Label(parent, SWT.NONE);
		priorityLabel.setText("Priority:");
	}

	private void createCCLabel(Composite parent) {
		ccButton = new Button(parent, SWT.NONE);
		ccButton.setText("Cc:");
	}


	private void createToButton(Composite parent) {
		ToButton = new Button(parent, SWT.NONE);
		ToButton.setText("To:");
	}

	private void createToText(Composite parent) {
	    GridData gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		

		ToText = new Text(parent, SWT.BORDER);
		ToText.setLayoutData(gridData);
		ToText.setText(mail.getReceivers());
		ToText
				.setToolTipText("Please input legal email address like abc@example.com, use \",\" or \";\" to separate two address");
		mailToAddress = mail.getReceivers();

		ToText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mailToAddress = ToText.getText();

			}
		});
	}

	private void createCotentsText(Composite parent) {

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
/*
 detailsText = new Text(parent, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
 | SWT.BORDER);*/
		detailsText = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		//gridLayout.horizontalSpacing = 5;
	//	detailsText.setLayout(gridLayout);
		detailsText.setLayoutData(gridData);
		
		//detailsText.setLayoutData(gridData);
		if (message.getMessageoperation() == ExtendedMessage.MessageOperation.Edited) {
			detailsText.setText(message.getPlainTextFromBodyPart());
			mailContent = detailsText.getText();
		} else {
			mailContent = "";
		}
		detailsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mailContent = detailsText.getText();
			}
		});
	}
	private void createCotentsTextFromHtml(Composite parent,String htmlContent) {
		message.setMessageFormat(Mail.MailFormat.PLAIN);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		detailsText = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		detailsText.setLayoutData(gridData);
	    mailContent = htmlContent;
	    detailsText.setText(mailContent);
		detailsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mailContent = detailsText.getText();
			}
		});
	}
	/**
	 * This method initializes ccText
	 * 
	 */
	private void createCCText(Composite parent) {
		   GridData gridData = new GridData();
			gridData.horizontalSpan = 4;
			gridData.horizontalAlignment = GridData.FILL;
			gridData.grabExcessHorizontalSpace = true;

		ccText = new Text(parent, SWT.BORDER);
		ccText.setLayoutData(gridData);
		ccText.setText(mail.getCcers());
		mailCcAddress = mail.getCcers();
		ccText
				.setToolTipText("Please input legal email address like abc@example.com, use \",\" or \";\" to separate two address");

		ccText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mailCcAddress = ccText.getText();
			}
		});
	}
	/**
	 * This method initializes priorityImageButton
	 * 
	 */
	private void createPriorityImageButton(final Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = false;

		priorityImageButton = new Button(parent, SWT.NONE);
		priorityImageButton.setLayoutData(gridData);
		String ImageString = null;
		if (mail.getPriority() == Priority.LOW)
			ImageString = Activator.IMAGE_MAIL_LOWPRI;
		else if (mail.getPriority() == Priority.NORMAL)
			ImageString = Activator.IMAGE_MAIL_NORMALPRI;
		else if (mail.getPriority() == Priority.HIGH)
			ImageString = Activator.IMAGE_MAIL_HIGHPRI;
		priorityImageButton.setImage(Activator.getDefault().getImage(
				ImageString));

		priorityImageButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				Menu m = new Menu(parent);
				MenuItem options1 = new MenuItem(m, SWT.CASCADE);
				options1.setText("High priority");
				options1.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent e) {

					}

					public void widgetSelected(SelectionEvent e) {
						priorityImageButton.setImage(Activator.getDefault()
								.getImage(Activator.IMAGE_MAIL_HIGHPRI));
						mail.setPriority(Priority.HIGH);

					}

				});
				MenuItem options2 = new MenuItem(m, SWT.CASCADE);
				options2.setText("Normal priority");
				options2.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent e) {

					}

					public void widgetSelected(SelectionEvent e) {
						priorityImageButton.setImage(Activator.getDefault()
								.getImage(Activator.IMAGE_MAIL_NORMALPRI));
						mail.setPriority(Priority.NORMAL);
					}

				});

				MenuItem options3 = new MenuItem(m, SWT.CASCADE);
				// TODO: Should add some functions here
				options3.setText("Low Priority");
				options3.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent e) {

					}

					public void widgetSelected(SelectionEvent e) {
						priorityImageButton.setImage(Activator.getDefault()
								.getImage(Activator.IMAGE_MAIL_LOWPRI));
						mail.setPriority(Priority.LOW);
					}

				});

				m.setVisible(true);

			}
		});

	}

	/**
	 * This method initializes subjectText
	 * 
	 */
	private void createSubjectText(Composite parent) {
		   GridData gridData = new GridData();
			gridData.horizontalSpan = 4;
			gridData.horizontalAlignment = GridData.FILL;
			gridData.grabExcessHorizontalSpace = true;

		subjectText = new Text(parent, SWT.BORDER);
		subjectText.setLayoutData(gridData);
		subjectText.setText(mail.getSubject());
		try {
			message.setSubject("");
		} catch (MessagingException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		subjectText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mail.setSubject(subjectText.getText());
				try {
					message.setSubject(subjectText.getText());
				} catch (MessagingException e1) {
					e1.printStackTrace();
				}

			}
		});
	}

	private void createAttachList(final Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		attachList = new List(parent, SWT.BORDER | SWT.V_SCROLL);
		attachList.setLayoutData(gridData);
		if (message.getMessageoperation() == ExtendedMessage.MessageOperation.Edited) {
			attachList.setItems(message.getAttachmentName());
		}
	
		attachList.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				final String selected[] = attachList.getSelection();
				Menu m = new Menu(parent);
				MenuItem options1 = new MenuItem(m, SWT.CASCADE);
				options1.setText("Delete from attachment(s)");
				options1.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						attachList.remove(selected[0]);
						message
								.removeAttachments(selected[0]);

					}

				});

				MenuItem options2 = new MenuItem(m, SWT.CASCADE);
				options2.setText("Open");
				options2.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						Program.launch(selected[0]);

					}

				});

				MenuItem options3 = new MenuItem(m, SWT.CASCADE);
				// TODO: Should add some functions here
				options3.setText("Property");
				m.setVisible(true);
			}
		});
	}
	
	public void readyForSaveInfo(){
		MailManager.getInstance().changeFolderIDByName(mail, "Draft");
		message.setMessageoperation(ExtendedMessage.MessageOperation.Edited);
		if (computeContent() != 0)
			return;
		mail.setCurrentDate();
		message.addAllsToMessage();
	}
	public void readyForSendInfo(){
		MailManager.getInstance().changeFolderIDByName(mail, "Outbox");
		message.setMessageoperation(ExtendedMessage.MessageOperation.Edited);
		if (computeContent() != 0)
			return;
		mail.setCurrentDate();
		message.addAllsToMessage();
	}


	private void createMenuBar() {
		m = new Menu(s, SWT.BAR);
		file = new MenuItem(m, SWT.CASCADE);
		filemenu = new Menu(s, SWT.DROP_DOWN);
		sendMenuItem = new MenuItem(filemenu, SWT.PUSH);
		saveMailMenuItem = new MenuItem(filemenu, SWT.PUSH);
		new MenuItem(filemenu, SWT.SEPARATOR);
		exitMenuItem = new MenuItem(filemenu, SWT.PUSH);
		edit = new MenuItem(m, SWT.CASCADE);
		editmenu = new Menu(s, SWT.DROP_DOWN);
		cutMenuItem = new MenuItem(editmenu, SWT.PUSH);
		copyMenuItem = new MenuItem(editmenu, SWT.PUSH);
		pasteMenuItem = new MenuItem(editmenu, SWT.PUSH);
		help = new MenuItem(m, SWT.CASCADE);
		helpmenu = new Menu(s, SWT.DROP_DOWN);
		aboutMenuItem = new MenuItem(helpmenu, SWT.PUSH);

		file.setText("&File");
		file.setMenu(filemenu);
		sendMenuItem.setText("&Open\tCTRL+O");
		sendMenuItem.setAccelerator(SWT.CTRL + 'O');
		saveMailMenuItem.setText("&Save\tCTRL+S");
		saveMailMenuItem.setAccelerator(SWT.CTRL + 'S');
		exitMenuItem.setText("E&xit");
		edit.setText("&Edit");
		edit.setMenu(editmenu);
		cutMenuItem.setText("&Cut");
		copyMenuItem.setText("Co&py");
		pasteMenuItem.setText("&Paste");
		help.setText("&Help");
		help.setMenu(helpmenu);
		aboutMenuItem.setText("&About");

		// add SelectionListeners for the menu items
		sendMenuItem.addSelectionListener(new SendMail());
		saveMailMenuItem.addSelectionListener(new SaveMail());
		exitMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				s.dispose();
			}
		});
		cutMenuItem.addSelectionListener(new Cut());
		copyMenuItem.addSelectionListener(new Copy());
		pasteMenuItem.addSelectionListener(new Paste());
		aboutMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				/*   AboutDialog ad = new AboutDialog(s);
				   ad.open( );*/
			}
		});

	}

	private void createShell() {

		s = new Shell(display, SWT.SHELL_TRIM);
		initTitle();
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		s.setLayout(layout);
		s.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				s.dispose();
			}
		});

	}

	private void createToolBar() {
		bar = new ToolBar(s, SWT.HORIZONTAL);
		//create ToolBar and ToolItems
		sendToolItem = new ToolItem(bar, SWT.PUSH);

		saveToolItem = new ToolItem(bar, SWT.PUSH);
		new ToolItem(bar, SWT.SEPARATOR).setWidth(20);
		attachmentToolItem = new ToolItem(bar, SWT.PUSH);
		new ToolItem(bar, SWT.SEPARATOR).setWidth(20);
		mailheadToolItem = new ToolItem(bar, SWT.DROP_DOWN);
		accountToolItem = new ToolItem(bar, SWT.DROP_DOWN);
		mailformatToolItem = new ToolItem(bar, SWT.DROP_DOWN);
		//set the size and location of the user interface widgets
		bar.setSize(500, 55);
		bar.setLocation(10, 0);

		//configure the toolbar            
		sendToolItem.setImage(sendIcon);
		sendToolItem.setText("Send");
		sendToolItem.setToolTipText("Send the mail");
		saveToolItem.setImage(saveIcon);
		saveToolItem.setText("Save");
		saveToolItem.setToolTipText("Save this mail in Draft folder");
		attachmentToolItem.setImage(attachIcon);
		attachmentToolItem.setText("attachment");
		attachmentToolItem.setToolTipText("Add attachment for this mail");
		mailheadToolItem.setImage(mailheaderIcon);
		mailheadToolItem.setText("Mail Head");
		mailheadToolItem.setToolTipText("Set mail header");
		accountToolItem.setImage(accountIcon);
		accountToolItem.setText("Account");
		accountToolItem.setToolTipText("Select account");
		mailformatToolItem.setImage(formatIcon);
		mailformatToolItem.setText("Format");
		mailformatToolItem.setToolTipText("Select format of this mail");

		//add SelectionListeners to the toolbar buttons          
		sendToolItem.addSelectionListener(new SendMail());
		saveToolItem.addSelectionListener(new SaveMail());
		attachmentToolItem.addSelectionListener(new AttachDropDownSelectionListener());
		mailheadToolItem.addSelectionListener(new MailHeadDropDownSelectionListener());
		accountToolItem.addSelectionListener(new AccountDropDownSelectionListener());
		mailformatToolItem.addSelectionListener(new FormatDownSelectionListener());
		}
	private void initTitle() {
		if (mail.getMailflag() == Mail.MailFlag.DRAFT) {
			s.setImage(Activator.getDefault().getImage(Activator.IMAGE_DRAFT));
			if (mail.isNewCreated()) {
				s.setText("Create a new mail");
			} else {
				s.setText("Edit:" + mail.getSubject());
			}

		} else {
			s.setImage(Activator.getDefault().getImage(Activator.IMAGE_MAIL_OPEN));
			if (mail.getSubject().equalsIgnoreCase("")) {
				s.setText("Mail");
			} else {
				s.setText(mail.getSubject());
			}
		}

	}

	public void show() {
		s.setMenuBar(m);
		s.setMaximized(true);
		s.open();
		while (!s.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		//display.dispose();
	}

	//create reusable named inner classes for SelectionListeners            
	class SendMail extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			readyForSendInfo();
			if(checkMailAddress())
				return;
			new SaveMessageJob(message,mail).schedule();
			SendMessageJob sendmessagejob = new SendMessageJob(mail, message);
			sendmessagejob.schedule();
			s.dispose();
		}

		private boolean checkMailAddress() {
			//TODO Need check address and hint user that fill out correct info
			return false;
			
		}
	}
	class AttachDropDownSelectionListener extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			FileDialog fd = new FileDialog(s, SWT.OPEN);
			fd.setText("Open");
			String selected = fd.open();
			attachList.add(selected);
			message.addAttachments(selected);
		}
	}
	class MailHeadDropDownSelectionListener extends SelectionAdapter {
		private Menu menu = null;
		private boolean visible = false;
		public void widgetSelected(SelectionEvent event) {
			// Create the menu if it has not already been created
			if (menu == null) {
				menu = new Menu(s);

				final MenuItem menuItem1 = new MenuItem(menu, SWT.NONE);
				if(isBccHide){
				menuItem1.setImage(Activator.getDefault().getImage(
						Activator.ACTIVITY_IMAGE_PATH));}
				menuItem1.setText("Bcc");

				menuItem1.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						isBccHide = isBccHide == true ? false:true;
						MailEditor.this.createBccButton(composite);
						MailEditor.this.createBccText(composite);
						s.layout();
					
					}
				});
			}
			// Position the menu below and vertically aligned with the the drop
			// down tool button.
			final ToolItem toolItem = (ToolItem) event.widget;
			final ToolBar toolBar = toolItem.getParent();

			Rectangle toolItemBounds = toolItem.getBounds();
			Point point = toolBar.toDisplay(new Point(toolItemBounds.x,
					toolItemBounds.y));
			menu.setLocation(point.x, point.y + toolItemBounds.height);
			setMenuVisible(true);

		}

		private void setMenuVisible(boolean visible) {
			menu.setVisible(visible);
			this.visible = visible;
		}
	}
	class AccountDropDownSelectionListener extends SelectionAdapter {
		private Menu menu = null;
		private boolean visible = false;

		public void widgetSelected(SelectionEvent event) {
			// Create the menu if it has not already been created
			if (menu == null) {
				// Lazy create the menu.
				menu = new Menu(s);
				int accountID[] = new int[MailManager.getInstance()
						.getAccounts().size()];
				accountID = MailManager.getInstance().getAccountsID();
				for (int i = 0; i < accountID.length; i++) {
					final MenuItem menuItem = new MenuItem(menu, SWT.NONE);
					if (accountID[i]==mail.getAccountID()) {
						menuItem.setImage(Activator.getDefault().getImage(
								Activator.ACTIVITY_IMAGE_PATH));
					}
					menuItem.setText(MailManager.getInstance().getAccountFromID(accountID[i]).getAccountName());

					menuItem.addSelectionListener(new SelectionAdapter() {
						public void widgetSelected(SelectionEvent e) {
							mailAccountName = menuItem.getText();
							mail.setAccountID(MailManager.getInstance().getAccountFromName(mailAccountName).getAccountID());
							menu.dispose();
							menu = null;
						}
					});
				}
			}
			// Position the menu below and vertically aligned with the the drop
			// down tool button.
			final ToolItem toolItem = (ToolItem) event.widget;
			final ToolBar toolBar = toolItem.getParent();

			Rectangle toolItemBounds = toolItem.getBounds();
			Point point = toolBar.toDisplay(new Point(toolItemBounds.x,
					toolItemBounds.y));
			menu.setLocation(point.x, point.y + toolItemBounds.height);
			setMenuVisible(true);

		}

		private void setMenuVisible(boolean visible) {
			menu.setVisible(visible);
			this.visible = visible;
		}
	}
	
	class FormatDownSelectionListener extends SelectionAdapter {
		private Menu menu = null;
		private boolean visible = false;
		public void widgetSelected(SelectionEvent event) {
			// Create the menu if it has not already been created
			if (menu == null) {
				menu = new Menu(s);

				final MenuItem menuItem1 = new MenuItem(menu, SWT.NONE);
				if(mail.getMAILFORMAT()==Mail.MailFormat.PLAIN){
				menuItem1.setImage(Activator.getDefault().getImage(
						Activator.ACTIVITY_IMAGE_PATH));}
				menuItem1.setText("Plain");

				menuItem1.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						if(mail.getMAILFORMAT()==Mail.MailFormat.PLAIN){
							
						}else{
							String htmlContent = MailEditor.this.html.getComposerConent();
							message.setMessageFormat(Mail.MailFormat.PLAIN);
							mail.setMAILFORMAT(Mail.MailFormat.PLAIN);
							for(int i=0 ,len = mailContentArea.getChildren().length; i< len; i++){
								mailContentArea.getChildren()[i].dispose();
							}
							 createCotentsTextFromHtml(mailContentArea,htmlContent);
							 mailContentArea.layout();
							
						
						}
						menu.dispose();
						menu = null;
					}
				
				});
				final MenuItem menuItem2 = new MenuItem(menu, SWT.NONE);
				if(mail.getMAILFORMAT()==Mail.MailFormat.HTML){
				menuItem2.setImage(Activator.getDefault().getImage(
						Activator.ACTIVITY_IMAGE_PATH));
				}
				menuItem2.setText("HTML");
				menuItem2.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						if (mail.getMAILFORMAT() == Mail.MailFormat.HTML) {
							
						} else {
							String tempContent = detailsText.getText();
							message.setMessageFormat(Mail.MailFormat.HTML);
							mail.setMAILFORMAT(Mail.MailFormat.HTML);
							for(int i=0 ,len = mailContentArea.getChildren().length; i< len; i++){
								mailContentArea.getChildren()[i].dispose();
							}
						   createHTMLEditorFromPlain(mailContentArea,tempContent);
						    mailContentArea.layout();
						}
						menu.dispose();
						menu = null;
					}
				});
			}
			// Position the menu below and vertically aligned with the the drop
			// down tool button.
			final ToolItem toolItem = (ToolItem) event.widget;
			final ToolBar toolBar = toolItem.getParent();

			Rectangle toolItemBounds = toolItem.getBounds();
			Point point = toolBar.toDisplay(new Point(toolItemBounds.x,
					toolItemBounds.y));
			menu.setLocation(point.x, point.y + toolItemBounds.height);
			setMenuVisible(true);

		}

		private void setMenuVisible(boolean visible) {
			menu.setVisible(visible);
			this.visible = visible;
		}
	}
	
	private void updateMailContentArea() {
		for(int i=0 ,len = mailContentArea.getChildren().length; i< len; i++){
			mailContentArea.getChildren()[i].dispose();
		}
	    this.createCotentsText(mailContentArea);
	    mailContentArea.layout();
		
	}
	

	private void updateToTextNew(Composite parent) {
		ToText.setParent(parent);
	    GridData gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		ToText.setLayoutData(gridData);
	}

	class SaveMail extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			readyForSaveInfo();
			new SaveMessageJob(message,mail).schedule();
			s.dispose();
		}
	}

	class Cut extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			contentText.cut();
		}
	}

	class Copy extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			contentText.copy();
		}
	}

	class Paste extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			contentText.paste();
		}
	}
	private int computeContent() {
		return processToAddress() + processCcAddress() + processContent()
				+ processAccount();

	}

	private int processAccount() {

		mail.setSender((MailManager.getInstance().getAccountFromID(
				mail.getAccountID()).getEmail()));
		mail.setAccountID(MailManager.getInstance().getAccountFromName(mailAccountName).getAccountID());
		message.setAccountID(mail.getAccountID());
		try {
			message.setFrom(new InternetAddress(MailManager.getInstance()
					.getAccountFromName(mailAccountName).getEmail()));
			message.setSender(new InternetAddress(MailManager.getInstance()
					.getAccountFromName(mailAccountName).getEmail()));
		} catch (MessagingException e) {
			Activator.logError(e);
			return 1;
		}
		return 0;
	}

	private int processContent() { 
		
		if(mail.getMAILFORMAT()==Mail.MailFormat.HTML){
		mailContent = html.getComposerConent();
		try {
			message.addHTMLContent(mailContent);
		} catch (MessagingException e) {
			Activator.logError(e);
			return 1;
		}
		}else{
			try {
				message.addTextContent(mailContent);
			} catch (MessagingException e) {
				Activator.logError(e);
				return 1;
			}
		}
		
		return 0;

	}

	private int processCcAddress() {
		String addresstemp = (mailCcAddress.toString()).replace(";", ",")
				.trim();
		mail.setCcers(addresstemp);
		try {
			message.setRecipients(Message.RecipientType.CC, addresstemp);
		} catch (MessagingException e) {

			Status status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e
					.getLocalizedMessage(), e);

			ErrorDialog.openError(s,
					"Please input correct mail address", e
							.getLocalizedMessage(), status);
			Activator.logError(e);
			return 1;

		}
		return 0;
	}

	private int processToAddress() {
		String addresstemp = (mailToAddress.toString()).replace(";", ",")
				.trim();
		mail.setReceivers(addresstemp);
		try {
			message.setRecipients(Message.RecipientType.TO, addresstemp);
		} catch (MessagingException e) {
			Activator.logError(e);
			Status status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e
					.getLocalizedMessage(), e);

			ErrorDialog.openError(s,
					"Please input correct mail address", e
							.getLocalizedMessage(), status);
			return 1;
		}
		return 0;
	}

}
