/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.editors;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

import javax.mail.MessagingException;

import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.models.Mail.Priority;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.actions.AddContactFromMailAction;
import org.eclipse.samples.organizer.ui.mail.jobs.DeleteMessageJob;
import org.eclipse.samples.organizer.ui.mail.main.ExtendedMessage;
import org.eclipse.samples.organizer.ui.mail.util.AttachmentCreation;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.*;

public class MailViewer {
	private Display display;
	private Shell s;
	private MenuItem aboutMenuItem;
	private MenuItem copyMenuItem;
	private Menu m;
	private MenuItem replyMenuItem;
	private Menu replymenu;
	private MenuItem exitMenuItem;
	private MenuItem edit;
	private Menu editmenu;
	private MenuItem cutMenuItem;
	private ToolBar bar;
	private Image forwardIcon;
	private Image replyAllIcon;
	private Image replyIcon;
	private Text contentText;
	private Image deleteIcon;
	private Image addContactIcon;
	private Image printIcon;
	private MenuItem help;
	private Menu helpmenu;
	private MenuItem pasteMenuItem;
	private MenuItem separator;
	private MenuItem replyAllMenuItem;
	private ToolItem deleteToolItem;
	private ToolItem addContactToolItem;
	private ToolItem sep1;
	private ToolItem replyToolItem;
	private ToolItem forwardToolItem;
	private ToolItem replyAllToolItem;
	private ToolItem printToolItem;
	private MenuItem file;

	private Label ccLabel = null;

	private Text ccText = null;

	private Label ToLabel = null;

	private Text ToText = null;

	private Label subjectLabel = null;

	private Text subjectText = null;

	private Text detailsText = null;

	private Label attachLabel;


	private Button priorityButton;
	private Label fromLabel;
	private Text fromText;
	private Text attachNoneText;
	private Label dateLabel;
	private Text dateText;

	private Mail mail;
	private ExtendedMessage message;
	private Composite composite;
	private MenuItem forwardMenuItem;
	private org.eclipse.swt.widgets.List attachList;

	public MailViewer(Mail _mail, ExtendedMessage _message) {
		this.mail = _mail;
		this.message = _message;
		display = Display.getDefault();
		initework();
		createShell();

		createMenuBar();
		createToolBar();
		createContentComposite();

	}

	private void initework() {
		// create images for toolbar buttons
		replyIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_BIGREPLY);
		replyAllIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_BIGREPLYALL);
		forwardIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_BIGFORWARD);
		deleteIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_DELETE);
		addContactIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_CONTACT);
		printIcon = Activator.getDefault().getImage(Activator.IMAGE_MAIL_PRINT);
	}

	private void createContentComposite() {
		composite = new Composite(s, SWT.MULTI);
		GridData spec = new GridData();
		spec.horizontalAlignment = GridData.FILL;
		spec.grabExcessHorizontalSpace = true;
		spec.verticalAlignment = GridData.FILL;
		spec.grabExcessVerticalSpace = true;
		composite.setLayoutData(spec);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.horizontalSpacing = 5;
		composite.setLayout(gridLayout);
		createContent(composite);

	}

	private void createContent(Composite parent) {
		createFromLabel(parent);
		createFromText(parent);

		createToLabel(parent);
		createToText(parent);

		if (mail.hasCcAddress()) {
			createCCLabel(parent);
			createCCText(parent);

		}
		createDateLabel(parent);
		createDateText(parent);
		createSubjectLabel(parent);
		if (mail.hasAttachment()) {
			createSubjectText(parent);
			createAttachLabel(parent);
			createAttachList(parent);
		} else {
			createSubjectTextSimple(parent);

		}
		createPriorityImage(parent);
		createCotentsText(parent);

	}

	private void createDateLabel(Composite parent) {
		dateLabel = new Label(parent, SWT.NONE);
		dateLabel.setText("Date:");
		
	}


	private void createFromLabel(Composite parent) {
		fromLabel = new Label(parent, SWT.NONE);
		fromLabel.setText("From:");
	}

	private void createAttachLabel(Composite parent) {
		attachLabel = new Label(parent, SWT.NONE);
		attachLabel.setText("Attachment:");
	}

	private void createSubjectLabel(Composite parent) {
		subjectLabel = new Label(parent, SWT.NONE);
		subjectLabel.setText("Subject:");
	}

	private void createCCLabel(Composite parent) {
		ccLabel = new Label(parent, SWT.NONE);
		ccLabel.setText("Cc:");
	}

	private void createToLabel(Composite parent) {
		ToLabel = new Label(parent, SWT.RIGHT);
		ToLabel.setText("To:");
	}

	private void createFromText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 4;
		gridData.grabExcessHorizontalSpace = true;

		fromText = new Text(parent, SWT.READ_ONLY);
		fromText.setLayoutData(gridData);
		fromText.setText(mail.getSender());
		fromText.setEditable(false);
	}

	private void createToText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		ToText = new Text(parent, SWT.READ_ONLY);
		ToText.setLayoutData(gridData);
		ToText.setText(mail.getReceivers());
	
	}

	private void createPriorityImage(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = false;

		priorityButton = new Button(parent, SWT.READ_ONLY);
		priorityButton.setLayoutData(gridData);
		String ImageString = null;
		if (mail.getPriority() == Priority.LOW)
			ImageString = Activator.IMAGE_MAIL_LOWPRI;
		else if (mail.getPriority() == Priority.NORMAL)
			ImageString = Activator.IMAGE_MAIL_NORMALPRI;
		else if (mail.getPriority() == Priority.HIGH)
			ImageString = Activator.IMAGE_MAIL_HIGHPRI;
		priorityButton.setImage(Activator.getDefault().getImage(ImageString));

	}

	private void createCotentsText(Composite parent) {
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;
		if (message.getMessageFormat() == Mail.MailFormat.PLAIN) {
			detailsText = new Text(parent, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
					| SWT.BORDER);
			detailsText.setLayoutData(gridData);
			if (!mail.isNewCreated())
				// TODO: Should consider different conditon.
				detailsText.setText(message.getPlainTextFromBodyPart());
			detailsText.setEditable(false);
		} else {
			Browser browser;
			try {
				browser = new Browser(parent, SWT.MULTI | SWT.WRAP
						| SWT.V_SCROLL | SWT.BORDER);
				browser.setLayoutData(gridData);
			} catch (SWTError e) {
				System.out.println("Could not instantiate Browser: "
						+ e.getMessage());
				return;
			}
			if(message.getPlainTextFromBodyPart()!=null)
			browser.setText(message.getPlainTextFromBodyPart());
		}
		/*
		 * shell.open(); while (!shell.isDisposed()) { if
		 * (!display.readAndDispatch()) display.sleep(); } display.dispose(); }
		 */
	}

	/**
	 * This method initializes ccText
	 * 
	 */
	private void createCCText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 4;
		gridData.grabExcessHorizontalSpace = true;

		ccText = new Text(parent, SWT.READ_ONLY);
		ccText.setLayoutData(gridData);
		ccText.setText(mail.getCcers());
	}

	private void createDateText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 4;
		gridData.grabExcessHorizontalSpace = true;

		dateText = new Text(parent, SWT.READ_ONLY);
		dateText.setLayoutData(gridData);
		try {
			dateText.setText(message.getSentDate().toLocaleString());
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	/**
	 * This method initializes subjectText
	 * 
	 */
	private void createSubjectText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 4;

		subjectText = new Text(parent, SWT.NONE);
		subjectText.setLayoutData(gridData);
		subjectText.setText(mail.getSubject());
		subjectText.setEditable(false);

	}
	private void createSubjectTextSimple(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		subjectText = new Text(parent, SWT.NONE);
		subjectText.setLayoutData(gridData);
		subjectText.setText(mail.getSubject());
		subjectText.setEditable(false);

	}
	private void createAttachList(final Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		if (message.getAttachmentName().length == 0) {
			attachNoneText = new Text(parent, SWT.READ_ONLY);
			attachNoneText.setLayoutData(gridData);
			return;
		}
		attachList = new org.eclipse.swt.widgets.List(parent, SWT.BORDER | SWT.V_SCROLL);
		attachList.setLayoutData(gridData);
		// if(!mail.isNewCreated())
		
		attachList.setItems(message.getAttachmentName());
		attachList.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				final String selected[] = attachList.getSelection();
				Menu m = new Menu(parent);

				MenuItem options1 = new MenuItem(m, SWT.CASCADE);
				options1.setText("Save as");
				options1.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent e) {

					}

					public void widgetSelected(SelectionEvent e) {
						FileDialog fd = new FileDialog(s, SWT.SAVE);
						fd.setText("Save");
						String fileselected = fd.open();
						File file = new File(selected[0]);
						if (fileselected != null && file.exists()) {
							try {
								AttachmentCreation.copy(selected[0],
										fileselected);
							} catch (Exception e1) {
								e1.printStackTrace();
							}
						}
					}

				});

				MenuItem options2 = new MenuItem(m, SWT.CASCADE);
				options2.setText("Open");
				options2.addSelectionListener(new SelectionListener() {

					public void widgetDefaultSelected(SelectionEvent e) {

					}

					public void widgetSelected(SelectionEvent e) {
						Program.launch(selected[0]);

					}

				});

				MenuItem options3 = new MenuItem(m, SWT.CASCADE);
				// TODO: Should add some functions here
				options3.setText("Property");
				m.setVisible(true);

			}
		});

	}
	

	private void createMenuBar() {
		m = new Menu(s, SWT.BAR);
		file = new MenuItem(m, SWT.CASCADE);
		replymenu = new Menu(s, SWT.DROP_DOWN);
		replyMenuItem = new MenuItem(replymenu, SWT.PUSH);
		replyAllMenuItem = new MenuItem(replymenu, SWT.PUSH);
		forwardMenuItem = new MenuItem(replymenu, SWT.PUSH);
		MenuItem separator = new MenuItem(replymenu, SWT.SEPARATOR);
		exitMenuItem = new MenuItem(replymenu, SWT.PUSH);
		edit = new MenuItem(m, SWT.CASCADE);
		editmenu = new Menu(s, SWT.DROP_DOWN);
		cutMenuItem = new MenuItem(editmenu, SWT.PUSH);
		copyMenuItem = new MenuItem(editmenu, SWT.PUSH);
		pasteMenuItem = new MenuItem(editmenu, SWT.PUSH);
		help = new MenuItem(m, SWT.CASCADE);
		helpmenu = new Menu(s, SWT.DROP_DOWN);
		aboutMenuItem = new MenuItem(helpmenu, SWT.PUSH);

		file.setText("&File");
		file.setMenu(replymenu);
		replyMenuItem.setText("&Reply\tCTRL+R");
		replyMenuItem.setAccelerator(SWT.CTRL + 'R');
		replyAllMenuItem.setText("ReplyAll");
		forwardMenuItem.setText("&Forward\tCTRL+F");
		forwardMenuItem.setAccelerator(SWT.CTRL + 'F');
		exitMenuItem.setText("E&xit");
		edit.setText("&Edit");
		edit.setMenu(editmenu);
		cutMenuItem.setText("&Cut");
		copyMenuItem.setText("Co&py");
		pasteMenuItem.setText("&Paste");
		help.setText("&Help");
		help.setMenu(helpmenu);
		aboutMenuItem.setText("&About");

		// add SelectionListeners for the menu items
		replyMenuItem.addSelectionListener(new ReplyMail());
		replyAllMenuItem.addSelectionListener(new ReplyAll());
		forwardMenuItem.addSelectionListener(new Forward());
		exitMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				s.dispose();
				s = null;
			}
		});
		cutMenuItem.addSelectionListener(new Cut());
		copyMenuItem.addSelectionListener(new Copy());
		pasteMenuItem.addSelectionListener(new Paste());
		aboutMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				/*   AboutDialog ad = new AboutDialog(s);
				   ad.open( );*/
			}
		});

	}

	private void createShell() {

		s = new Shell(display, SWT.SHELL_TRIM);
		initTitle();
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		s.setLayout(layout);
		s.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				s.dispose();
				s = null;
			}
		});

	}

	private void createToolBar() {
		bar = new ToolBar(s, SWT.HORIZONTAL);
		//create ToolBar and ToolItems
		replyToolItem = new ToolItem(bar, SWT.PUSH);
		replyAllToolItem = new ToolItem(bar, SWT.PUSH);
		forwardToolItem = new ToolItem(bar, SWT.PUSH);
		new ToolItem(bar, SWT.SEPARATOR).setWidth(20);
		addContactToolItem = new ToolItem(bar, SWT.PUSH);
		printToolItem  = new ToolItem(bar, SWT.PUSH);
		deleteToolItem = new ToolItem(bar, SWT.PUSH);
		//set the size and location of the user interface widgets
		bar.setSize(500, 55);
		bar.setLocation(10, 0);

		//configure the toolbar            
		replyToolItem.setImage(replyIcon);
		replyToolItem.setText("Reply");
		replyToolItem.setToolTipText("Reply the mail");
		replyAllToolItem.setImage(replyAllIcon);
		replyAllToolItem.setText("ReplyAll");
		replyAllToolItem.setToolTipText("Reply to all");
		forwardToolItem.setImage(forwardIcon);
		forwardToolItem.setText("Forward");
		forwardToolItem.setToolTipText("Forward this mail");
		
		addContactToolItem.setImage(addContactIcon);
		addContactToolItem.setText("Contact");
		addContactToolItem.setToolTipText("Add contact");
		printToolItem.setImage(printIcon);
		printToolItem.setText("Print");
		printToolItem.setToolTipText("Print prevous mail");
		deleteToolItem.setImage(deleteIcon);
		deleteToolItem.setText("Delete");
		deleteToolItem.setToolTipText("Delete the Mail mail");
		//add SelectionListeners to the toolbar buttons          
		replyToolItem.addSelectionListener(new ReplyMail());
		replyAllToolItem.addSelectionListener(new ReplyAll());
		forwardToolItem.addSelectionListener(new Forward());
		deleteToolItem.addSelectionListener(new Delete());
		addContactToolItem.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				String allAddresses = ((new StringBuilder(mail.getSender())).append(",").append(mail.getReceivers()).append(",").append(mail.getCcers()).append(",").append(mail.getBccers())).toString();
				new AddContactFromMailAction(allAddresses).run();
			}
		});

	}

	private void initTitle() {
		if (mail.getMailflag() == Mail.MailFlag.DRAFT) {
			s.setImage(new Image(display, Activator.IMAGE_DRAFT));
			if (mail.isNewCreated()) {
				s.setText("Create a new mail");
			} else {
				s.setText("Edit draft:" + mail.getSubject());
			}

		} else {
			s.setImage(Activator.getDefault().getImage(Activator.IMAGE_MAIL_OPEN));
			if (mail.getSubject().equalsIgnoreCase("")) {
				s.setText("Mail");
			} else {
				s.setText(mail.getSubject());
			}
		}

	}

	public void show() {
		s.setMenuBar(m);
		s.setBounds(150, 100, 1000, 800);
		s.open();
		while (!s.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		//display.dispose();
	}

	//create reusable named inner classes for SelectionListeners            
	class ReplyMail extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			message.setMessageoperation(ExtendedMessage.MessageOperation.Reply);
            new MailReplyForwardEditor(mail, message).show();
            s.dispose();
            s = null;
		}
	}

	class ReplyAll extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {message.setMessageoperation(ExtendedMessage.MessageOperation.ReplyToAll);
        new MailReplyForwardEditor(mail, message).show();
        s.dispose();
        s = null;}
	}
	class Forward extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {message.setMessageoperation(ExtendedMessage.MessageOperation.Forward);
        new MailReplyForwardEditor(mail, message).show();
        s.dispose();
        s = null;}
	}

	class Cut extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			contentText.cut();
		}
	}


class Copy extends SelectionAdapter {
	public void widgetSelected(SelectionEvent event) {
		contentText.copy();
	}
}
	class Delete extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			List<Mail> mailList = new ArrayList();
			mailList.add(mail);
	     	new DeleteMessageJob(mailList).schedule();
	     	s.dispose();
	     	s =null ;
		}
	}

	class Paste extends SelectionAdapter {
		public void widgetSelected(SelectionEvent event) {
			contentText.paste();
		}
	}

}
