/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.helper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.*;
import java.util.ArrayList;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.events.ContentChangeEvent;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.listeners.ContentChangeListener;
import org.eclipse.samples.organizer.mails.filters.AccountFilter;
import org.eclipse.samples.organizer.mails.filters.MailFilterByFolderID;
import org.eclipse.samples.organizer.mails.filters.MailFilterByUnread;
import org.eclipse.samples.organizer.main.MailFolder;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.actions.CreateMailAction;
import org.eclipse.samples.organizer.ui.mail.actions.CreateNewAccountAction;
import org.eclipse.samples.organizer.ui.mail.actions.DeleteAccountAction;
import org.eclipse.samples.organizer.ui.mail.actions.DeleteFolderAction;
import org.eclipse.samples.organizer.ui.mail.actions.EmptyFolderAction;
import org.eclipse.samples.organizer.ui.mail.actions.PropertyChangeAction;
import org.eclipse.samples.organizer.ui.mail.actions.ReceiveMailAction;
import org.eclipse.samples.organizer.ui.mail.actions.SendMailOutBoxAction;
import org.eclipse.samples.organizer.ui.model.FeatureHelper;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

public class MailViewFeatureHelper implements FeatureHelper {
	

	TreeViewer viewer;

	private ContentChangeListener contentChangeListener = new ContentChangeListener() {
		public void contentChanged(ContentChangeEvent event) {
			viewer.refresh();
		}
	};

	class ViewContentProvider implements IStructuredContentProvider,
			ITreeContentProvider, ISelectionChangedListener {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (parent instanceof MailFolder) {
				return getChildren(parent);
			}
			return new Object[0];
		}

		public Object getParent(Object child) {
			if (child instanceof MailFolder) {
				return ((MailFolder) child).getParent();
			}
			return null;
		}

		public Object[] getChildren(Object parent) {
			if (parent instanceof MailFolder) {
				return ((MailFolder) parent).getChildren();
			}
			return new Object[0];
		}

		public boolean hasChildren(Object parent) {
			if (parent instanceof MailFolder)
				return ((MailFolder) parent).hasChildren();
			return false;
		}

		public void selectionChanged(SelectionChangedEvent event) {
			IStructuredSelection ss = (IStructuredSelection) event
					.getSelection();
			if (ss.getFirstElement() instanceof MailFolder) {
				MailFolder foderItem = (MailFolder) ss.getFirstElement();
				if (foderItem.isAccount()) {
					MailManager.getInstance().setCurrentAccountFromID(foderItem.getAccountID());
				} else {
					try {
                      
						MailManager.getInstance().setCurrentFolder(
								(MailFolder) ss.getFirstElement());
						
						MailFilterByFolderID.instance.setFolderID(MailManager.getInstance()
								.getCurrentFolder().getFolderID());
						Activator
								.getDefault()
								.getWorkbench()
								.getActiveWorkbenchWindow()
								.getActivePage()
								.showView(
										"org.eclipse.samples.organizer.ui.mail.views.MailView");
						//Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart().setFocus();
					} catch (PartInitException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	class ViewLabelProvider extends LabelProvider {

		public String getText(Object obj) {
			if (obj instanceof MailFolder) {
				if (((MailFolder) obj).getUnreadMessageCount() == 0)
					return ((MailFolder) obj).getFolderName();
				else
					return ((MailFolder) obj).getFolderName() + "("
							+ ((MailFolder) obj).getUnreadMessageCount() + ")";
			}

			return obj.toString();
		}

		public Image getImage(Object obj) {
			String imageKey = ISharedImages.IMG_OBJ_ELEMENT;
			if (obj instanceof MailFolder) {
				if (((MailFolder) obj).isAccount()) {
					return Activator.getDefault().getImage(Activator.ACCOUNT_IMAGE_PATH);
				} else if (((MailFolder) obj).getFolderName().equalsIgnoreCase(
						"Inbox")) {
					return Activator.getDefault().getImage(Activator.INBOX_IMAGE_PATH);
				} else if (((MailFolder) obj).getFolderName().equalsIgnoreCase(
						"Outbox")) {
					return Activator.getDefault().getImage(Activator.OUTBOX_IMAGE_PATH);
				} else if (((MailFolder) obj).getFolderName()
						.equalsIgnoreCase("Sent")) {
					return Activator.getDefault().getImage(Activator.SENT_IMAGE_PATH);
				} else if (((MailFolder) obj).getFolderName().equalsIgnoreCase(
						"Trash")) {
					return Activator.getDefault().getImage(Activator.TRASH_IMAGE_PATH);
				} else if (((MailFolder) obj).getFolderName().equalsIgnoreCase(
						"Draft")) {
					return Activator.getDefault().getImage(Activator.DRAFT_IMAGE_PATH);
				}

				imageKey = ISharedImages.IMG_OBJ_FOLDER;
			}
			return PlatformUI.getWorkbench().getSharedImages().getImage(
					imageKey);
		}
	}

	public void createSwitcherContentArea(final Composite parent) {
        
		parent.setLayout(new FillLayout());

		MailManager.getInstance().addContentChangeListener(
				contentChangeListener);
		try {
			if (OrganizerManager.instance.count(AccountFilter.instance)==0) {
				new CreateNewAccountAction(parent.getShell()).run();
			}
		} catch (BackendException e1) {
			e1.printStackTrace();
		}
	/*	final PropertyChangeListener filterCompleteChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
			}
		};
		MailFilter.instance.addPropertyChangeListener(MailFilter.FOLDER_ID, filterCompleteChangeListener);*/
		ViewContentProvider vcp = new ViewContentProvider();
		viewer = new TreeViewer(parent, SWT.NONE);
		viewer.addSelectionChangedListener(vcp);
		viewer.setContentProvider(vcp);

		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setInput(getMailFolderRoot());
		createActions(parent);
		try {
			Activator.getDefault().getWorkbench().showPerspective("org.eclipse.samples.organizer.ui.mail.perspective", Activator.getDefault().getWorkbench().getActiveWorkbenchWindow());
		} catch (WorkbenchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	private void createActions(final Composite parent) {
		viewer.getTree().addMouseListener(new MouseAdapter() {

			public void mouseDown(MouseEvent e) {
				if (e.button != 3)
					return;
				if (!(((Tree) e.getSource()).getSelection()[0].getData() instanceof MailFolder))
					return;
				final MailFolder tempFolder = (MailFolder) ((Tree) e
						.getSource()).getSelection()[0].getData();
				if (!tempFolder.isAccount()) {
					Menu m = new Menu(viewer.getControl());
					MenuItem options1 = new MenuItem(m, SWT.CASCADE);
					options1.setText("Empty folder");
					options1.addSelectionListener(new SelectionAdapter() {

						public void widgetSelected(SelectionEvent e) {
							MessageBox messageBox = new MessageBox(parent
									.getShell(), SWT.ICON_QUESTION | SWT.YES
									| SWT.NO);
							messageBox
									.setMessage("Do you really want to delete all mails under this folder?");
							messageBox.setText("Empty folder confirm Dialog");
							int response = messageBox.open();
							if (response == SWT.YES) {
								new EmptyFolderAction(
										tempFolder.getFolderID()).run();
							}
						}

					});
					MenuItem options2 = new MenuItem(m, SWT.CASCADE);
					options2.setText("Delete folder");
					options2.addSelectionListener(new SelectionAdapter() {

						public void widgetSelected(SelectionEvent e) {
							MessageBox messageBox = new MessageBox(parent
									.getShell(), SWT.ICON_QUESTION | SWT.YES
									| SWT.NO);
							messageBox
									.setMessage("Do you really want to delete athis folder?");
							messageBox.setText("Delete folder confirm Dialog");
							int response = messageBox.open();
							if (response == SWT.YES) {
								new DeleteFolderAction(
										tempFolder.getFolderID()).run();

							}
						}
					});
					if(tempFolder.isSystemfoldersymbol())
					options2.setEnabled(false);
					m.setVisible(true);
					return;
				}
				
				Menu m = new Menu(viewer.getControl());
				
				MenuItem options3 = new MenuItem(m, SWT.CASCADE);
				options3.setText("Composing mail");
				options3.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
							 new CreateMailAction(new ArrayList<String>()).run();
					}

				});
				
				MenuItem options1 = new MenuItem(m, SWT.CASCADE);
				options1.setText("Receive mails");
				options1.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						
							IAction receiveMailForAccountAction = new ReceiveMailAction(
									tempFolder.getAccountID());
							receiveMailForAccountAction.run();

						
						IAction receiveAction = new ReceiveMailAction(
								tempFolder.getAccountID());
						receiveAction.run();
					}

				});

				MenuItem options2 = new MenuItem(m, SWT.CASCADE);
				options2.setText("Send mails in Outbox");
				options2.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						IAction sendMailOutBoxAction = new SendMailOutBoxAction(tempFolder
								.getAccountID());
						sendMailOutBoxAction.run();
					}

				});
				
				new MenuItem(m, SWT.SEPARATOR);
				MenuItem options4 = new MenuItem(m, SWT.CASCADE);
				options4.setText("Create new account");
				options4.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						IAction createnewaccountaction = new CreateNewAccountAction(Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell()
								);
						createnewaccountaction.run();
					}

				});
				MenuItem options5 = new MenuItem(m, SWT.CASCADE);
				options5.setText("Delete account");
				options5.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						MessageBox messageBox = new MessageBox(parent
								.getShell(), SWT.ICON_QUESTION | SWT.YES
								| SWT.NO);
						messageBox.setMessage("Do you really want to delete this account");
						messageBox.setText("Delete confirm");
						int response = messageBox.open();
						if (response == SWT.YES) {
							IAction deleteAccountAction = new DeleteAccountAction(
									tempFolder.getAccountID());
							deleteAccountAction.run();

						}

					}

				});
				new MenuItem(m, SWT.SEPARATOR);
				MenuItem options7 = new MenuItem(m, SWT.CASCADE);
				options7.setText("Property");
				options7.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						IAction propertyAction = new PropertyChangeAction(
								parent.getDisplay().getActiveShell(), viewer, tempFolder
										.getAccountID());
						propertyAction.run();
					}

				});
				m.setVisible(true);
			}

		});

	}

	private MailFolder getMailFolderRoot() {
		MailFolder root = new MailFolder();

		for (int i = 0; i < MailManager.getInstance().getAccounts().size(); i++) {
			root
					.add(MailManager.getInstance().getAccounts().get(i)
							.getFolder());
		}
		return root;
	}

	public void configureSwitcherButton(Button button) {

	}

	public String getSummary() {
		MailFilterByUnread.instance.setUnreadCheck(true);
		return String.valueOf(getMail()) + " unread mails";
	}
	
	private int getMail() {
		try {
			return OrganizerManager.instance.count(MailFilterByUnread.instance);
		} catch (BackendException e) {
			return 0;
		}
	}
}
