/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.main.MessageOperations;
import org.eclipse.swt.widgets.Display;

public class DeleteMessageJob extends BasicJob {

	private List<Mail> mailList;
	private int len;

	public DeleteMessageJob(List<Mail> s) {
		super("Delete mails", s.get(0).getAccountID());
		mailList = s;
		len = mailList.size();
	}

	@Override
	protected IStatus run(final IProgressMonitor monitor) {
		
		final int trashfolderID = MailManager.getInstance().getFolderIDByString("Trash");
		//Just move mail to trash folder
		if(mailList.get(0).getFolderID()!=trashfolderID)
		{
			monitor.beginTask("Moving mail(s) to Trash folder", len) ;
			for(int i=0;i<len;i++){
				if(monitor.isCanceled()) break; 
				final Mail mail = mailList.get(i);
				monitor.worked(i);
				Display.getDefault().syncExec(new Runnable() {
		            public void run() {
		            	mail.setFolderIDToDataBase(trashfolderID);
		            }
				});
			}
			monitor.done();
			return Status.OK_STATUS;
		} 
		//Delete mail content from database and disk;
		else {
			monitor.beginTask("Delete mail perpetually", len);
			for (int i = 0; i < len; i++) {
				if (monitor.isCanceled())
					break;
				
				if (MessageOperations.deleteMessage(mailList.get(i)
						.getDataFilePath())) {
					monitor.worked(i);

				} else {
                 //TODO, hint user that some message can't be deleted from disk.
				}
			}
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					MailManager.getInstance().deleteMailsFromDatabase(mailList);
				}
			});
			monitor.done();
		}
		return Status.OK_STATUS;
	}

}
