/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import java.io.IOException;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.URLName;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.ui.mail.jobs.BasicJob;
import org.eclipse.samples.organizer.ui.mail.main.MessageOperations;

import com.sun.mail.pop3.POP3Folder;

public class Pop3GetAndDelFromServerJob extends BasicJob{


	public  Pop3GetAndDelFromServerJob(int id) {
		super("Receiving mails from " + MailManager.getInstance().getAccountFromID(id).getAccountName(), id);
	}
	
	@Override
	public IStatus run(IProgressMonitor monitor) {
		try {
			downloadMessages(monitor);
		} catch (NoSuchProviderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BackendException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (monitor.isCanceled())
			return Status.CANCEL_STATUS;

		return Status.OK_STATUS;

	}

	private void downloadMessages(IProgressMonitor monitor) throws NoSuchProviderException,
			BackendException {
		POP3Folder inbox = null;
		URLName url = new URLName("pop3", this.getReceiveServerAddress(), this
				.getReceiveServerPort(), "", this.getEmailAdress(), this
				.getEmailPassword());
		
		Session session = Session.getInstance(System.getProperties(), null);
		Store store = session.getStore(url);
		try {
			store.connect();
			inbox = (POP3Folder) store.getFolder("INBOX");
			inbox.open(Folder.READ_WRITE);
			Message[] messages = inbox.getMessages();
			int messagelen = messages.length;
			monitor.beginTask("Downloading mail from server...",messagelen);

			for (int i = 0; i < messagelen; i++) {
				if (monitor.isCanceled())
					return;
				monitor.worked(i);
				receiveNewMessage(messages[i]);
				messages[i].setFlag(Flags.Flag.DELETED,true);
			}
			inbox.close(true); // Delete mails on server!
			store.close();// close
		} catch (Exception e) {
			e.printStackTrace();
		}
		monitor.done();
	}
	private void receiveNewMessage(final Message message)
			throws MessagingException, IOException {

		try {
			String uid = "New";
			MessageOperations.createMailWithMessage(message, account, uid);
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
