/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.mail.FetchProfile;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.UIDFolder;
import javax.mail.URLName;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.ui.mail.main.MessageOperations;

import com.sun.mail.pop3.POP3Folder;

public class Pop3GetJob extends BasicJob {
	private List<String> mailUID = new ArrayList<String>();

	public Pop3GetJob(int id) {
		super("Receiving mails from " + MailManager.getInstance().getAccountFromID(id).getAccountName(), id);
	}

	@Override
	public IStatus run(IProgressMonitor monitor) {
		try {
			downloadMessages(monitor);
		} catch (NoSuchProviderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BackendException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (monitor.isCanceled())
			return Status.CANCEL_STATUS;

		return Status.OK_STATUS;

	}

	private void downloadMessages(IProgressMonitor monitor) throws NoSuchProviderException,
			BackendException {
		MailManager.getInstance().fetchMailUIDs(mailUID,MailManager.getInstance().getFolderIDByString("Inbox"));
		POP3Folder inbox = null;
		boolean match = false;
		URLName url = new URLName("pop3", this.getReceiveServerAddress(), this
				.getReceiveServerPort(), "", this.getEmailAdress(), this
				.getEmailPassword());
		
		Session session = Session.getInstance(System.getProperties(), null);
		Store store = session.getStore(url);
		try {
			store.connect();
			inbox = (POP3Folder) store.getFolder("INBOX");
			inbox.open(Folder.READ_ONLY);
			FetchProfile profile = new FetchProfile();
			profile.add(UIDFolder.FetchProfileItem.UID);
			//  profile.add(FetchProfile.Item.ENVELOPE);
			Message[] messages = inbox.getMessages();
			inbox.fetch(messages, profile);
			
			int messagelen = messages.length;
			monitor.beginTask("Downloading mail from server...",messagelen);
			String[] messageUID = new String[messagelen];
			
			for (int i = 0; i < messagelen; i++) {
				if(monitor.isCanceled()) return; 
				monitor.worked(i);
				match = false;
				messageUID[i] = (inbox.getUID(messages[i]));
				for (int j = 0; j < mailUID.size(); j++) {
					if (messageUID[i].equalsIgnoreCase(mailUID.get(j))) {
						match = true;
						continue;
					}
				}
				if (!match) {
					receiveNewMessage(messages[i], messageUID[i]);
				}
			}
			inbox.close(false);   //Not delete the mails on server!
			store.close();//close 
		} catch (Exception e) {
			e.printStackTrace();
		}
		monitor.done();
	}

	private void receiveNewMessage(final Message message, final String uid)
			throws MessagingException, IOException {

		try {
			MessageOperations.createMailWithMessage(message, account, uid);
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
