/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.mail.Message;
import javax.mail.MessagingException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.main.ExtendedMessage;
import org.eclipse.swt.widgets.Display;

public class SaveMessageJob extends BasicJob{

	private Mail mail;
	private ExtendedMessage message;


	public SaveMessageJob(ExtendedMessage _message,Mail _mail) {
		super("Saving mail", _mail.getAccountID());
		this.mail= _mail;
		this.message = _message;
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		monitor.beginTask("Saving mail:"+mail.getSubject(), 100);
		createNewMail(message, mail,monitor);
		monitor.done();
		return Status.OK_STATUS;
	}
	
	private void createNewMail(Message message,final Mail mail,IProgressMonitor monitor){
		File file = new File(mail.getDataFilePath());
		try {
			file.createNewFile();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		monitor.worked(10);
		try {
			message.writeTo(new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		monitor.worked(70);
		setMailSize(mail,message);
		setAttachmentInfo(mail,message);
		Display.getDefault().syncExec(new Runnable() {
            public void run() {
            	MailManager.getInstance().addSingleMailToDatabase(mail);
            }
		});
	}
	private  void setAttachmentInfo(Mail mail, Message message) {
		if(message instanceof ExtendedMessage){
			if(((ExtendedMessage)message).hasAttachment())
			mail.setAttachments(1);
		}
		
	}

	private  void setMailSize(Mail mail,Message message){
		try {
			mail.setSize(message.getSize());
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
