/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.main.ExtendedMessage;

public class SendMailInOutBoxJob extends BasicJob {
   
	private List<Mail> mailList = new ArrayList<Mail>();
	public SendMailInOutBoxJob(int id) {
		super("Sending mails in outbox from " + MailManager.getInstance().getAccountFromID(id).getAccountName(), id);
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		int outboxID = MailManager.getInstance().getFolderIDByString("Outbox");
		try {
			MailManager.getInstance().fetchMails(mailList, outboxID);
		} catch (BackendException e) {
			e.printStackTrace();
		}
		if(mailList.size()==0){
			Activator.getDefault().showMessageOnStatusbar("There is no message in outbox");
		}
		else{
			for(int i =0; i < mailList.size(); i++){
				if(mailList.get(i).getMailflag()!= Mail.MailFlag.DRAFT)
					continue;
				ExtendedMessage message = new ExtendedMessage();
			    message.setAccountID(mailList.get(i).getAccountID());
				message.getMessageFromFile(mailList.get(i).getDataFilePath());
				new SendMessageJob(mailList.get(i), message).schedule();
			}
			
		}
		return Status.OK_STATUS;
	}

}
