/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.jobs;

import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.MimeMessage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.main.MessageOperations;
import org.eclipse.swt.widgets.Display;

public class SendMessageJob extends BasicJob {
	private MimeMessage message;
	private Mail mail;

	public SendMessageJob(Mail mail, MimeMessage message) {
		super("Sending mail:" + mail.getSubject(), mail.getAccountID());
		this.message = message;
		this.mail = mail;

	}

	@Override
	public IStatus run(IProgressMonitor monitor) {
		boolean sendSuccess = false;
		monitor.beginTask("Sending mail" + mail.getSubject(), 100);
		if(MailManager.getInstance().getFolderIDByString("Outbox")==mail.getFolderID())
		{
			
		}
		else{
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					MailManager.getInstance().changeFolderIDByNameAndSave(mail,
							"Outbox");
				}
			});
		}

		Session session = MessageOperations.createSendSession();
		try {
			Transport transport = session.getTransport();
			transport.connect(this.getSMTPAddress(), this.getEmailAdress(),
					this.getEmailPassword());
			monitor.worked(20);
			transport.sendMessage(message, message.getAllRecipients());
			sendSuccess = true;
			transport.close();

			monitor.done();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (sendSuccess)
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						MailManager.getInstance().changeFolderIDByNameAndSave(
								mail, "Sent");
					}
				});
		}
		return Status.OK_STATUS;
	}
}
