/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.mail.Address;
import javax.mail.BodyPart;
import javax.mail.Flags;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.samples.organizer.ui.mail.util.MailPathProducer;

public class ExtendedMessage extends MimeMessage {

	public enum MessageOperation{Default,Reply,ReplyToAll,Forward,Edited};
	private Mail.MailFormat messageFormat;
	
	private MessageOperation messageoperation = MessageOperation.Default;

	private int accountID;
	

	private Map<String, MimeBodyPart> attachMentBodyPart = new HashMap<String, MimeBodyPart>();

	// The String text of Text and HTML content, both are supported
	private MimeBodyPart plainContentBodyPart = new MimeBodyPart();
	private MimeBodyPart htmlContentBodyPart = new MimeBodyPart();
/*
 * private String ContentType;
 */

/* private MessageStatus messageStaus; */

	Multipart multipart = new MimeMultipart();

	public boolean hasAttachment() {
		if (attachMentBodyPart.isEmpty())
			return false;
		return true;
	}

	public String getPlainTextFromBodyPart() {
		try {
			return (String) plainContentBodyPart.getContent();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";
	}
	public String getHTMLFromBodyPart() {
		try {
			return (String) htmlContentBodyPart.getContent();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return "";
	}
	

	public String[] getAttachmentName() {
		if (attachMentBodyPart.isEmpty())
			return new String[] {};
		return attachMentBodyPart.keySet().toArray(
				new String[attachMentBodyPart.keySet().size()]);

	}

	public ExtendedMessage(Session session) {
		super(session);
	}

	public ExtendedMessage() {
		super(MessageOperations.createSendSession());
	}

	/**
	 * Set the flags for this message.
	 */
	public void setFlag(Flags.Flag flag, boolean set) throws MessagingException {
		if (super.isSet(flag) != set) {
			super.setFlag(flag, set);

		}
	}

	public void getMessageFromFile(String filename) {
		try {
			this.parse(new FileInputStream(filename));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			Activator.logError("The file of this message can't be found");
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		getMessageInfoFromObject();
	}

/*
 * public MessageStatus getMessageStaus() { return messageStaus; }
 * 
 * public void setMessageStaus(MessageStatus messageStaus) { this.messageStaus =
 * messageStaus; }
 */

	private void addContentToMessage() {
        if (this.getMessageFormat() == Mail.MailFormat.PLAIN) {
			try {
				multipart.addBodyPart(plainContentBodyPart);
			} catch (MessagingException e) {
				e.printStackTrace();
			}
		}else{
			try {
				multipart.addBodyPart(htmlContentBodyPart);
			} catch (MessagingException e) {
				e.printStackTrace();
			}
		}
	}

	private void addAttachmentsToMessage() {

		for (Iterator<MimeBodyPart> i = attachMentBodyPart.values().iterator(); i
				.hasNext();)
			try {
				multipart.addBodyPart(i.next());
			} catch (MessagingException e) {
				e.printStackTrace();
			}

	}

	private void getMessageInfoFromObject() {
		List<String> fileName = new ArrayList<String>();
		Object content = null;
		try {
			content = this.getContent();
		} catch (IOException e1) {
			Activator.getDefault().showErrorMessageOnStatusbar(e1.getMessage());
			Activator.logError(e1);
		} catch (MessagingException e1) {
			Activator.logError(e1);
		}
		if (content instanceof Multipart) {
			multipart = (Multipart) content;
			try {
				for (int i = 0; i < this.multipart.getCount(); i++) {
					BodyPart bp = this.multipart.getBodyPart(i);
					if (bp.getDisposition() != null) {
						if (bp.getDisposition().equals(Part.INLINE)) {
							plainContentBodyPart.setText((String) bp.getContent());
						} else {
							File tempFile = new File(
									MailPathProducer.DEFAULT_ATTACHMENT_FOLDER
											+ bp.getFileName());
							try {
								tempFile.createNewFile();
							} catch (IOException e) {
								e.printStackTrace();
							}
							attachMentBodyPart.put(tempFile.getAbsolutePath(),
									(MimeBodyPart) bp);
							try {
								((MimeBodyPart) bp).saveFile(tempFile);
							} catch (FileNotFoundException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							fileName.add(tempFile.getAbsolutePath());
						}
					}

					else if (bp.isMimeType("text/html")) {
						this.setMessageFormat(Mail.MailFormat.HTML);
						htmlContentBodyPart.setText((String) bp.getContent());
					} else if (bp.isMimeType("text/plain")) {
						this.setMessageFormat(Mail.MailFormat.PLAIN);
						plainContentBodyPart.setText((String) bp.getContent());
					} else if (bp.isMimeType("multipart/*")) {

						Multipart secondMuiltipart = (Multipart) bp
								.getContent();
						for (int j = 0; j < secondMuiltipart.getCount(); j++) {
							BodyPart secondBodyPart = secondMuiltipart
									.getBodyPart(j);
							if (secondBodyPart.getDisposition() != null) {
								File tempFile = new File(
										MailPathProducer.DEFAULT_ATTACHMENT_FOLDER
												+ secondBodyPart.getFileName());
								try {
									tempFile.createNewFile();
								} catch (IOException e) {
									e.printStackTrace();
								}
								attachMentBodyPart.put(tempFile
										.getAbsolutePath(),
										(MimeBodyPart) secondBodyPart);
								try {
									((MimeBodyPart) secondBodyPart)
											.saveFile(tempFile);
								} catch (FileNotFoundException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
								fileName.add(tempFile.getAbsolutePath());
							}

							else if (secondBodyPart.isMimeType("text/html")) {
								this.setMessageFormat(Mail.MailFormat.HTML);
								htmlContentBodyPart.setText((String) secondBodyPart
										.getContent());
							} else if (secondBodyPart.isMimeType("text/plain")) {
								this.setMessageFormat(Mail.MailFormat.PLAIN);
								plainContentBodyPart.setText((String) secondBodyPart
										.getContent());
							} else {
								// Rate is low
							}

						}
					} else {
						System.out.println("The " + bp.getContentType());
					}

				}
			} catch (MessagingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		// Not a Mime Message;
		else {
			this.setMessageFormat(Mail.MailFormat.PLAIN);
			try {
				// System.out.println(content);System.out.print(content.toString());
				plainContentBodyPart.setText(content.toString());
			} catch (MessagingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}


	public void addAllsToMessage() {
		/*
		 * if(ContentType.equalsIgnoreCase("text/plain")) {} else{
		 *  }
		 */
		addContentToMessage();
		addAttachmentsToMessage();
		try {
			this.setContent(multipart);
			
		} catch (MessagingException e1) {
			e1.printStackTrace();
			Activator.getDefault().showErrorMessageOnStatusbar(e1.getMessage());
		}
		try {
			this.saveChanges();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
	}

	public void addTextContent(String content) throws MessagingException {
		plainContentBodyPart.setText(content);
	}
	public void addHTMLContent(String content) throws MessagingException{
		this.setMessageFormat(Mail.MailFormat.HTML);
		htmlContentBodyPart.setContent(content,"text/html");
	}

	public void addAttachments(String filename) {
		MimeBodyPart attachmentTemp = MailAttachmentDeal
				.createAttachment(filename);
		attachMentBodyPart.put(filename, attachmentTemp);
	}

	public void removeAttachments(String filename) {
		if (attachMentBodyPart.containsKey(filename))
			attachMentBodyPart.remove(filename);
	}

	/*
	 * public String getMessageFilePath() { return messageFilePath; }
	 */

	public void setAccountID(int accountID) {
		this.accountID = accountID;
	}

	public int getAccountID() {
		return accountID;
	}

	public Mail.MailFormat getMessageFormat() {
		return messageFormat;
	}

	public void setMessageFormat(Mail.MailFormat messageFormat) {
		this.messageFormat = messageFormat;
	}

	public MessageOperation getMessageoperation() {
		return messageoperation;
	}

	public void setMessageoperation(MessageOperation messageoperation) {
		this.messageoperation = messageoperation;
	}


	@Override
	public void setRecipients(javax.mail.Message.RecipientType arg0,
			Address[] arg1) {
		try {
			super.setRecipients(arg0, arg1);
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void setSender(Address arg0) throws MessagingException {
		// TODO Auto-generated method stub
		super.setSender(arg0);
	}

	@Override
	public void setSubject(String arg0) throws MessagingException {
		// TODO Auto-generated method stub
		super.setSubject(arg0);
	}

	@Override
	public void setRecipients(javax.mail.Message.RecipientType arg0, String arg1) throws MessagingException {
		
			super.addRecipients(arg0, arg1);
		
	}
	
	public void getReplyForwardCotent(){
		try {
			plainContentBodyPart.setText((transferReplyForwardMessage((String)plainContentBodyPart.getContent())));
		} catch (IOException e) {
			e.printStackTrace();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
	}
	
	private String transferReplyForwardMessage(String content){
	    String enterChar = "\r\n";
	    String symbolChar = "\r\n>";
		StringBuilder finalConent = new StringBuilder();
			try {
				finalConent.append(this.getFrom()[0].toString() + " wrote on " + this.getSentDate().toLocaleString());
			} catch (MessagingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			finalConent.append(enterChar);
			finalConent.append(content);
		content = enterChar + enterChar + finalConent.toString().replaceAll(enterChar, symbolChar);
		return content;
		
	}

}
