/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.main;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Message.RecipientType;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.models.Mail;
import org.eclipse.samples.organizer.ui.mail.util.MailPathProducer;
import org.eclipse.swt.widgets.Display;

public class MessageOperations {
    private static Session defaultSession;
    
    public static Session getDefaultSession() {
        
        return Session.getDefaultInstance(new Properties());
    }
    
    public static Session createSendSession(){
    	
    	Properties props = new Properties();
    	props.setProperty("mail.transport.protocol", "smtp");
    	props.setProperty("mail.smtp.auth", "true");
    	Session session = Session.getInstance(props);
    	session.setDebug(true);
    	return session;
    	
    }
    
    public static void createMailWithMessage(Message message,MailAccount account,String uid) throws MessagingException, IOException{
    	Mail mail = new Mail();
    	mail.setUid(uid);
    	mail.setSize(message.getSize());
    	mail.setRead(0);
    	mail.setDateFromOutSide(message.getSentDate());
    	mail.setAccountID(account.getAccountID());
    	MailManager.getInstance().changeFolderIDByName(mail, "Inbox");
    	mail.setReceivers(changeAccountToString(message.getRecipients(RecipientType.TO)));
    	mail.setCcers(changeAccountToString(message.getRecipients(RecipientType.CC)));
    	mail.setBccers(changeAccountToString(message.getRecipients(RecipientType.BCC)));
    	mail.setSender(changeAccountToString(message.getFrom()));
    	mail.setSubject(message.getSubject());
    	mail.setMailflag(Mail.MailFlag.RECENT);
    	mail.setMAILFORMAT(setMailFormatByContentType(message.getContentType()));
    	mail.setAttachments(existAttachment(message)?1:0);
    	MailPathProducer.producePath(mail,"Inbox");
    	createNewMail(message,mail);
    }
    
	private static void createNewMail(Message message,final Mail mail){
		File file = new File(mail.getDataFilePath());
		try {
			file.createNewFile();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		try {
			message.writeTo(new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		setMailSize(mail,message);
		setAttachmentInfo(mail,message);
		Display.getDefault().syncExec(new Runnable() {
            public void run() {
            	MailManager.getInstance().addSingleMailToDatabase(mail);
            }
		});
		
	}
    public static String changeAccountToString(Address[] address){
    	if(address==null)
    		return "";
    	StringBuilder addressString = new StringBuilder();
    	for(int i=0;i<address.length;i++){
    		addressString.append(address[i].toString());
    	}
    	return addressString.toString();
    }
    
    public static Mail.MailFormat setMailFormatByContentType(String contentType) {
		if(contentType.startsWith("multi")){
			return Mail.MailFormat.HTML;
		}
    	return Mail.MailFormat.PLAIN;
	}
    /**
     * Perhaps this method should be more investigated, because it can deal with common sceniro.but not complicated things
     * @param message
     * @return
     * @throws IOException
     * @throws MessagingException
     */
    public static boolean existAttachment(Message message) throws IOException,
			MessagingException {
		Object content = message.getContent();
		if (content instanceof Multipart) {
			Multipart mp = (Multipart) content;
			for (int i = 0; i < mp.getCount(); i++) {
				Part part = mp.getBodyPart(i);
				String disposition = part.getDisposition();
				if ((disposition != null)
						&& ((disposition.equals(Part.ATTACHMENT) || (disposition
								.equals(Part.INLINE))))) {
					return true;
				}
			}

		}
		return false;

	}
    
	
	public static boolean deleteMessage(String filePath){
		return (new File("filePath")).delete();
	}
	private static void setAttachmentInfo(Mail mail, Message message) {
		if(message instanceof ExtendedMessage){
			if(((ExtendedMessage)message).hasAttachment())
			mail.setAttachments(1);
		}
		
	}

	private static void setMailSize(Mail mail,Message message){
		try {
			mail.setSize(message.getSize());
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
