/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.properties;

import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.main.MailFolder;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;

public class AccountInfoProperty extends PropertyPage {
	private MailAccount account;
	private Label accountNameLabel = null;
	private Text accountNameText = null;
	private Label mailAddressLabel = null;
	private Text mailAddressText = null;
	private Label passwordLabel = null;
	private Text passwordText = null;

	@Override
	protected Control createContents(Composite parent) {
		int accounid = ((MailFolder) this.getElement()
				.getAdapter(MailFolder.class)).getAccountID();
		account = MailManager.getInstance().getAccountFromID(accounid);
		initialize(parent);
		return parent;
	}

	private void initialize(Composite composite) {
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.grabExcessHorizontalSpace = true;

		accountNameLabel = new Label(composite, SWT.NONE);
		accountNameLabel.setText("Account Name:");

		accountNameText = new Text(composite, SWT.BORDER);
		accountNameText.setLayoutData(gridData);
		accountNameText.setText(account.getAccountName());
		accountNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setAccountName(accountNameText.getText());
			}
		});

		mailAddressLabel = new Label(composite, SWT.NONE);
		mailAddressLabel.setText("Mail Address:");

		mailAddressText = new Text(composite, SWT.BORDER);
		mailAddressText.setLayoutData(gridData);
		mailAddressText.setText(account.getEmail());
		mailAddressText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setEmail(mailAddressText.getText());
			}
		});

		passwordLabel = new Label(composite, SWT.NONE);
		passwordLabel.setText("New password:");

		passwordText = new Text(composite, SWT.PASSWORD | SWT.BORDER);
		passwordText.setLayoutData(gridData);
		passwordText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setPassword(passwordText.getText());
			}
		});

	}

}
