/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.properties;

import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.main.MailFolder;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPropertyPage;
import org.eclipse.ui.dialogs.PropertyPage;

public class MailServerProperty extends PropertyPage implements
		IWorkbenchPropertyPage {
	private MailAccount account;
	private Combo receiveTypeCombo = null;
	private Text receiveServerAddress = null;
	private Label smtpServerLabel = null;
	private Text smptServerAddress = null;
	private Label receiveLabel;
	private Label sendLabel;
	private Text receiveServerPort;
	private Label receiveServerPortLabel;
	private Label sendServerPortLabel;
	private Text sendServerPort;
	public MailServerProperty() {
		// TODO Auto-generated constructor stub
	}

	@Override
	protected Control createContents(Composite parent) {
		int accounid = ((MailFolder) this.getElement()
				.getAdapter(MailFolder.class)).getAccountID();
		account = MailManager.getInstance().getAccountFromID(accounid);
		initialize(parent);
		return parent;
	}
	private void initialize(Composite composite) {
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.grabExcessHorizontalSpace = true;
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
	
		receiveLabel = new Label(composite, SWT.READ_ONLY);
		receiveLabel.setText("Receiving server Info:");

		receiveTypeCombo = new Combo(composite, SWT.READ_ONLY);
		receiveTypeCombo.add("Pop3 Address:");
		receiveTypeCombo.add("IMAP addrss:");
		receiveTypeCombo.select(0);
		receiveTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if(receiveTypeCombo.getSelectionIndex()==0)
				  account.setIncomingtype("Pop3");
				else 
				   account.setIncomingtype("IMAP");
			}
		});
	
		receiveServerAddress = new Text(composite, SWT.BORDER);
		receiveServerAddress.setLayoutData(gridData);
		receiveServerAddress.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setIncomingAddress(receiveServerAddress.getText());
			}
		});
		receiveServerPortLabel = new Label(composite, SWT.NONE);
		receiveServerPortLabel.setText("Receving server port:(The default value is 110)");
		
		receiveServerPort = new Text(composite, SWT.BORDER);
		receiveServerPort.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setIncomingAddress(receiveServerAddress.getText());
			}
		});
		
		sendLabel = new Label(composite, SWT.READ_ONLY);
		sendLabel.setText("Sending server Info:");
		
		smtpServerLabel = new Label(composite, SWT.NONE);
		smtpServerLabel.setText("SMTP Address: ");
		
		smptServerAddress = new Text(composite, SWT.BORDER);
		smptServerAddress.setLayoutData(gridData);
		smptServerAddress.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setOutgoingAddress(smptServerAddress.getText());
			}
		});
		
		sendServerPortLabel = new Label(composite, SWT.NONE);
		sendServerPortLabel.setText("Receving server port:(The default value is 25)");
		
		sendServerPort = new Text(composite, SWT.BORDER);
		sendServerPort.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setIncomingAddress(receiveServerAddress.getText());
			}
		});
	}
	}

