/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.wizards;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.samples.organizer.main.MailManager;
import org.eclipse.samples.organizer.ui.mail.Activator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

public class ImportAccountWizard extends Wizard implements INewWizard {
	MailAccount account;
	ImportAccountWizardPage iaw ;
	private Backend backend;
	private List<Backend> backends;
	
	public ImportAccountWizard(MailAccount account) {
		super();
		this.account = account;
		iaw = new ImportAccountWizardPage(account);
		this.backends = getBackends();
		this.backend = getDefaultBackend();
		addPage(iaw);
		
	}
	
	/*private ImportAccountWizard(){
		super();
		this.account = new MailAccount();
		iaw = new ImportAccountWizardPage(account);
		addPage(iaw);
	}*/

	private List<Backend> getBackends() {
		return OrganizerManager.instance.getBackends();
	}
	public static void importTasks(final Shell shell, final MailAccount account) {
		
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ImportAccountWizard wizard = new ImportAccountWizard(account);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.setTitle("New Mail Account");
				dialog.create();
				dialog.open();
			}
		});
	}
	
	@Override
	public Image getDefaultPageImage() {
		return Activator.getDefault().getImage("/icons/Account_new.png");
	}
	@Override
	public boolean canFinish() {
		return backend != null;
	}
	@Override
	public  boolean performFinish() {
		if(isDuplicateAccount())
		{
			MessageBox messageBox = new MessageBox(getShell(),
					SWT.ICON_QUESTION | SWT.OK);
			messageBox
					.setMessage("Mail Account Name or Mail address already exists!");
			messageBox.setText("Duplicate Account Name or Mail Adress");
			messageBox.open();
			return false;
		}
		OrganizerManager.instance.setPreferredTaskBackend(backend);
		try {
			List<MailAccount> accountList = new ArrayList<MailAccount>();
			accountList.add(account);
			OrganizerManager.instance.add(backend, accountList);
			MailManager.getInstance().getAccounts().add(account);
		} catch (Exception e) {
			showMessage("An error occurred while attempting to complete the operation. More information has be written to the log file.");
			Activator.logError(e);
		}
		return true;
	}
	private Backend getDefaultBackend() {
		assert backends != null;
		assert backends.size() > 0;
		
		Backend preferred = OrganizerManager.instance.getPreferredAccountBackend();
		if (preferred == null) return backends.get(3);
		return preferred;
	}
	
	private void showMessage(String message) {
		MessageDialog.openInformation(getShell(), "Import Tasks Wizard", message);
	}

/*	private boolean isNotValidValue() {
		MailAccount acc = iaw.getAccount();
		if(acc.getAccountName()==null||account.getAccountName().equalsIgnoreCase("")||account.getEmail()==null ||account.getEmail().endsWith(""))
		return true;
		else
		return false;
	}*/

	private boolean isDuplicateAccount() {
		List<MailAccount> accounts = MailManager.getInstance().getAccounts();

		for (int i = 0, len = accounts.size(); i < len; i++) {
			// if account is already exist
			if (account.getAccountName().equalsIgnoreCase(
					accounts.get(i).getAccountName())
					|| account.getEmail().equalsIgnoreCase(
							accounts.get(i).getEmail())) {
				
				return true;
			}
		}
		return false;
	}

	

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		 setWindowTitle("New Mail Accout");
	}
	
/*	 public void addPage(IWizardPage page) {
		 addPage(new ImportAccountWizardPage(account));
	    }*/
	
	

}
