/*******************************************************************************
* Copyright (c) 2007 Ha Li.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Ha Li - initial API and implementation
*/
package org.eclipse.samples.organizer.ui.mail.wizards;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.samples.organizer.main.MailAccount;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class ImportAccountWizardPage extends WizardPage {

	private final MailAccount account;

	private Label accountNameLabel = null;
	private Text accountNameText = null;
	private Label mailAddressLabel = null;
	private Text mailAddressText = null;
	private Label passwordLabel =null;
	private Text passwordText = null;
	private Combo receiveTypeCombo = null;
	private Text receiveServerAddress = null;
	private Label smtpServerLabel = null;
	private Text smptServerAddress = null;


	private void initialize(Composite composite) {
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.grabExcessHorizontalSpace = true;
		
		
		accountNameLabel = new Label(composite, SWT.NONE);
		accountNameLabel.setText("Account Name:");
		
		
		accountNameText = new Text(composite, SWT.BORDER);	
		accountNameText.setLayoutData(gridData);
		accountNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setAccountName(accountNameText.getText());
				updateWizardErrorMessage();
			}
		});
		
	
		
		mailAddressLabel = new Label(composite, SWT.NONE);
		mailAddressLabel.setText("Mail Address:");
		
		mailAddressText = new Text(composite, SWT.BORDER);
		mailAddressText.setLayoutData(gridData);
		mailAddressText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setEmail(mailAddressText.getText());
				updateWizardErrorMessage();
			}
		});
		
		passwordLabel = new Label(composite, SWT.NONE);
		passwordLabel.setText("Password:");
		
		passwordText = new Text(composite, SWT.PASSWORD|SWT.BORDER);
		passwordText.setLayoutData(gridData);
		passwordText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setPassword(passwordText.getText());
				updateWizardErrorMessage();
			}
		});
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
	
		
		composite.setLayout(gridLayout);
		createReceiveTypeCombo(composite);
	//	setSize(new Point(423, 292));
		receiveServerAddress = new Text(composite, SWT.BORDER);
		receiveServerAddress.setLayoutData(gridData);
		receiveServerAddress.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setIncomingAddress(receiveServerAddress.getText());
				updateWizardErrorMessage();
			}
		});
		
		smtpServerLabel = new Label(composite, SWT.NONE);
		smtpServerLabel.setText("SMTP Address: ");
		
		smptServerAddress = new Text(composite, SWT.BORDER);
		smptServerAddress.setLayoutData(gridData);
		smptServerAddress.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				account.setOutgoingAddress(smptServerAddress.getText());
				updateWizardErrorMessage();
			}
		});
	
	}

	/**
	 * This method initializes receiveTypeCombo	
	 * @param composite 
	 *
	 */
	private void createReceiveTypeCombo(Composite composite) {
		receiveTypeCombo = new Combo(composite, SWT.READ_ONLY);
		receiveTypeCombo.add("Pop3 Address:");
		receiveTypeCombo.add("IMAP addrss:");
		receiveTypeCombo.select(0);
		receiveTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if(receiveTypeCombo.getSelectionIndex()==0)
				  account.setIncomingtype("Pop3");
				else 
				   account.setIncomingtype("IMAP");
			}
		});
	}

	protected ImportAccountWizardPage(MailAccount account) {
        
		super("New mail Accout!");
		this.account = account;
		
	}
	

	public void createControl(Composite parent) {
		setDescription("Create a New Account!  Please fill out following blanks!");
		//this.setImageDescriptor(Activator.imageDescriptorFromPlugin(Activator.getSymbolicName(), Activator.ACCOUNT_IMAGE_PATH));
		Composite composite =  new Composite(parent, SWT.NONE);
		initialize(composite);
		setControl(composite);
	}
	public IWizardPage getNextPage() {
		return getWizard().getNextPage(this);
	}

	public boolean canFlipToNextPage() {
		return getNextPage() != null;
	}

	public MailAccount getAccount() {
		return account;
	}
	
    protected void updateWizardErrorMessage(){
    	String accountName = account.getAccountName();
    	String mailAddress = mailAddressText.getText();
    	String receiverServerAddress = receiveServerAddress.getText();
    	String password = passwordText.getText();
    	
    	String errorMes = null;
    	if(accountName == null || accountName.equalsIgnoreCase(""))
    	{
    		errorMes = "Account Name could not be empty!";
    	}else if(mailAddress == null || mailAddress.equalsIgnoreCase(""))
    	{
    		errorMes = "Email Address could not be empty!";
    	}else if(!mailAddress.contains("@")){
    		errorMes = "Email Addess format is not right, please fill as good@example.com";
    	}else if(password == null || password.equalsIgnoreCase("")){
    		errorMes = "Password could not be empty!";
    	}else if(receiverServerAddress ==null ||receiverServerAddress.equalsIgnoreCase("")){
    		
    		errorMes = "Receive Servier Address could not be empty!";
    	}
    	setErrorMessage(errorMes);
    	this.setPageComplete(errorMes ==null);
    	}
    

	
}
