package org.eclipse.samples.organizer.ui.model;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.plugin.AbstractUIPlugin;

public class Feature {

	private final IConfigurationElement configurationElement;
	private final String text;
	private final String mainViewId;
	private ImageDescriptor image16Descriptor;
	private ImageDescriptor image32Descriptor;
	private ImageDescriptor image48Descriptor;
	private FeatureHelper featureHelper;

	public Feature(IConfigurationElement element) {
		configurationElement = element;
		text = element.getAttribute("text");
		mainViewId = element.getAttribute("mainViewId");
		image16Descriptor = getImageDescriptor("image16");
		image32Descriptor = getImageDescriptor("image32");
		image48Descriptor = getImageDescriptor("image48");
	}

	private ImageDescriptor getImageDescriptor(String key) {
		String imagePath = configurationElement.getAttribute(key);
		if (imagePath == null) return null;
		return AbstractUIPlugin.imageDescriptorFromPlugin(configurationElement.getContributor().getName(), imagePath);
	}

	public String getText() {
		return text;
	}

	public ImageDescriptor getImage16Descriptor() {
		return image16Descriptor;
	}
	
	public ImageDescriptor getImage32Descriptor() {
		return image32Descriptor;
	}

	public ImageDescriptor getImage48Descriptor() {
		return image48Descriptor;
	}
	
	public String getMainViewId() {
		return mainViewId;
	}

	public void createSwitcherContentArea(Composite parent) {
		getFeatureHelper().createSwitcherContentArea(parent);
	}

	private FeatureHelper getFeatureHelper() {
		if (featureHelper == null) featureHelper = createFeatureHelper();
		return featureHelper;
	}
	
	private FeatureHelper createFeatureHelper() {
		Object object;
		try {
			object = configurationElement.createExecutableExtension("helper");
			if (object instanceof FeatureHelper) return (FeatureHelper)object;
		} catch (CoreException e) {
			// TODO Log this?
		}
		return new DefaultFeatureHelper();
	}

	public void configureSwitcherButton(Button button) {
		getFeatureHelper().configureSwitcherButton(button);
	}
	

	public String getPerspectiveId() {
		return configurationElement.getAttribute("perspective");
	}

	public String getSummary() {
		return getFeatureHelper().getSummary();
	}

}
