package org.eclipse.samples.organizer.ui.switcher;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.samples.organizer.ui.model.Feature;
import org.eclipse.samples.organizer.ui.model.OrganizerViewModel;
import org.eclipse.samples.organizer.ui.model.OrganizerViewModel.SwitcherStyle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.part.ViewPart;

public class SwitcherView extends ViewPart {
	
	private Font buttonFont;
	private Font labelFont;
	private Composite switcherArea;
	private ImageRegistry imageRegistry;
	private Label imageLabel;
	private Label textLabel;
	private Composite labelArea;
	private Composite contentArea;
	private Label summaryLabel;
	
	@Override
	public void createPartControl(Composite parent) {
		allocateResources(parent);
				
		int inset = 2;
		
		final FormLayout layout = new FormLayout();
		layout.marginHeight = inset;
		layout.marginWidth = inset;
		
		parent.setLayout(layout);
			
		labelArea = new Composite(parent, SWT.NONE);
		createLabelArea();
				
		contentArea = new Composite(parent, SWT.NONE);
		contentArea.setLayout(new FillLayout());
		new TreeViewer(contentArea, SWT.NONE);
		
		switcherArea = new Composite(parent, SWT.NONE);
		createSwitcherArea();

		FormData labelAreaFormData = new FormData();
		labelAreaFormData.top = new FormAttachment(0, inset);
		labelAreaFormData.left = new FormAttachment(0, inset);
		labelAreaFormData.right = new FormAttachment(100, -inset);
		labelArea.setLayoutData(labelAreaFormData);
		
		FormData contentAreaFormData = new FormData();
		contentAreaFormData.top = new FormAttachment(labelArea, inset);
		contentAreaFormData.left = new FormAttachment(0, inset);
		contentAreaFormData.bottom = new FormAttachment(switcherArea, -inset);
		contentAreaFormData.right = new FormAttachment(100, -inset);		
		contentArea.setLayoutData(contentAreaFormData);
		
		FormData switcherAreaFormData = new FormData();
		switcherAreaFormData.left = new FormAttachment(0, inset);
		switcherAreaFormData.bottom = new FormAttachment(100, -inset);
		switcherAreaFormData.right = new FormAttachment(100, -inset);		
		switcherArea.setLayoutData(switcherAreaFormData);	
		
		hookChangeListeners(parent);

		updateContentArea();
	}

	private void createLabelArea() {
		final Color backgroundColor = labelArea.getDisplay().getSystemColor(SWT.COLOR_DARK_GRAY);
		final Color textColor = labelArea.getDisplay().getSystemColor(SWT.COLOR_WHITE);
		labelArea.setBackground(backgroundColor);
		
		int inset = 2;		
		FormLayout layout = new FormLayout();
		layout.marginHeight = inset;
		layout.marginWidth = inset;
		
		labelArea.setLayout(layout);
		
		imageLabel = new Label(labelArea, SWT.CENTER);
		imageLabel.setBackground(backgroundColor);
		
		textLabel = new Label(labelArea, SWT.LEFT);
		textLabel.setFont(labelFont);
		textLabel.setBackground(backgroundColor);
		textLabel.setForeground(textColor);
		
		summaryLabel = new Label(labelArea, SWT.LEFT);
		summaryLabel.setBackground(backgroundColor);
		summaryLabel.setForeground(textColor);
		summaryLabel.setText("15 tasks");
		
		FormData imageFormData = new FormData();
		imageFormData.top = new FormAttachment(0, inset);
		imageFormData.left = new FormAttachment(0, inset);
		
		imageLabel.setLayoutData(imageFormData);

		FormData textFormData = new FormData();
		textFormData.top = new FormAttachment(0, inset);
		textFormData.left = new FormAttachment(imageLabel, inset);
		textFormData.right = new FormAttachment(100, -inset);
		
		textLabel.setLayoutData(textFormData);

		FormData summaryFormData = new FormData();
		summaryFormData.top = new FormAttachment(textLabel, inset);
		summaryFormData.left = new FormAttachment(imageLabel, inset);
		summaryFormData.right = new FormAttachment(100, -inset);
		
		summaryLabel.setLayoutData(summaryFormData);
		updateLabelArea();
	}


	protected void updateLabelArea() {
		Feature feature = getActiveFeature();
		textLabel.setText(feature.getText());	
		imageLabel.setImage(getImage48(feature));	
		summaryLabel.setText(feature.getSummary());
	}
	
	protected void updateContentArea() {
		disposeAllChildren(contentArea);
		getActiveFeature().createSwitcherContentArea(contentArea);
		contentArea.layout();
	}
	
	private void createSwitcherArea() {
		disposeAllChildren(switcherArea);
		
		Feature[] features = getOrganizerViewModel().getFeatures();
		
		if (isSwitcherWithButtonsAndText()) 
			createSwitcherAsButtonsWithText(features);
		else 
			createSwitcherAsButtons(features);
		
		switcherArea.layout();
		switcherArea.getParent().layout();
	}

	private void disposeAllChildren(Composite parent) {
		Control[] children = parent.getChildren();
		for(int index=0;index<children.length;index++)
			children[index].dispose();
	}

	private boolean isSwitcherWithButtonsAndText() {
		return getSwitcherStyle() == OrganizerViewModel.SwitcherStyle.ButtonsWithText;
	}

	private void createSwitcherAsButtons(Feature[] features) {
		switcherArea.setLayout(new RowLayout());
		for (final Feature feature : features) {			
			final Button button = new Button(switcherArea, SWT.TOGGLE);
			button.setImage(getImage32(feature));
			configureButton(feature, button);
		}
	}

	private void createSwitcherAsButtonsWithText(Feature[] features) {
		switcherArea.setLayout(new GridLayout(1, true));
		for (Feature feature : features) {			
			Button button = new Button(switcherArea, SWT.TOGGLE);
			button.setText(feature.getText());
			button.setImage(getImage32(feature));
			button.setFont(buttonFont);
			button.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			configureButton(feature, button);
		}
	}
	
	/**
	 * This method integrates the given button with the {@link OrganizerViewModel}.
	 * Specifically, this method sets up two listeners. The first listener
	 * changes the selection state of the button to reflect the active feature;
	 * the button will be toggled on if the feature given in the parameter
	 * matches the current feature in the view model, or toggled off if it doesn't match.
	 * When the user clicks on the given button, the view model's feature is set
	 * to the feature given in the parameter.
	 * @param feature the feature represented by the button.
	 * @param button the button.
	 */
	private void configureButton(final Feature feature, final Button button) {
		if (getOrganizerViewModel().getActiveFeature() == feature) {
			button.setSelection(true);
		}
		
		final PropertyChangeListener listener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				button.setSelection(event.getNewValue() == feature);
			}				
		};
		getOrganizerViewModel().addPropertyChangeListener(OrganizerViewModel.ACTIVE_FEATURE, listener);
		
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				getOrganizerViewModel().setActiveFeature(feature);
				button.setSelection(true);
			}			
		});
		
		// When the button is disposed, remove the property change listener
		// from the view model.
		button.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				getOrganizerViewModel().removePropertyChangeListener(OrganizerViewModel.ACTIVE_FEATURE, listener);
			}				
		});
		
		feature.configureSwitcherButton(button);
	}

	private Feature getActiveFeature() {
		return getOrganizerViewModel().getActiveFeature();
	}
	
	private SwitcherStyle getSwitcherStyle() {
		return getOrganizerViewModel().getSwitcherStyle();
	}

	private OrganizerViewModel getOrganizerViewModel() {
		return OrganizerViewModel.instance;
	}
	
	/**
	 * This method returns a 32x32 image for the given {@link Feature}.
	 * This images must not be disposed explicitly; it will be
	 * automatically be disposed when this view closes.
	 */
	private Image getImage32(Feature feature) {
		return getImage(feature.getImage32Descriptor());
	}

	/**
	 * This method returns a 48x48 image for the given {@link Feature}.
	 * This images must not be disposed explicitly; it will be
	 * automatically be disposed when this view closes.
	 */
	private Image getImage48(Feature feature) {
		return getImage(feature.getImage48Descriptor());
	}

	/**
	 * This method returns the image described by the given descriptor.
	 * An image will only actually be allocated once and is stored in
	 * an {@link ImageRegistry}. The image will be disposed when the
	 * receiver closes, and so should not be explicitly disposed.
	 */
	private Image getImage(ImageDescriptor descriptor) {
		if (descriptor == null) return null;
		String key = descriptor.toString();
		Image image = imageRegistry.get(key);
		if (image == null) {
			imageRegistry.put(key, descriptor);
			image = imageRegistry.get(key);
		}
		return image;
	}

	private void allocateResources(Composite parent) {
		buttonFont = new Font(parent.getDisplay(),"Arial", 12, SWT.NORMAL);
		labelFont = new Font(parent.getDisplay(),"Arial", 14, SWT.NORMAL);
		imageRegistry = new ImageRegistry(parent.getDisplay());
		// Add a dispose listener so that when the view is destroyed
		// the allocated resources are released
		parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				buttonFont.dispose();			
				labelFont.dispose();
				imageRegistry.dispose();
			}			
		});
	}

	private void hookChangeListeners(Composite parent) {
		// Add a listener for changes to the SwitcherStyle.
		final PropertyChangeListener switcherStyleListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				createSwitcherArea();
			}
		};		
		getOrganizerViewModel().addPropertyChangeListener(OrganizerViewModel.SWITCHER_STYLE, switcherStyleListener);
		
		// Add a listener for changes to the mode.
		final PropertyChangeListener activeFeatureChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateLabelArea();
				updateContentArea();
			}
		};		
		getOrganizerViewModel().addPropertyChangeListener(OrganizerViewModel.ACTIVE_FEATURE, activeFeatureChangeListener);

		// Add a dispose listener so that when the view is destroyed
		// the listeners are removed.
		parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				getOrganizerViewModel().removePropertyChangeListener(OrganizerViewModel.SWITCHER_STYLE, switcherStyleListener);
				getOrganizerViewModel().removePropertyChangeListener(OrganizerViewModel.ACTIVE_FEATURE, activeFeatureChangeListener);
			}			
		});
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}

}
