/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.dnd.text;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.ui.task.dnd.DropHandler;
import org.eclipse.samples.organizer.ui.util.StringUtils;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;

public class TextDropHandler implements DropHandler {
	private TextTransfer textTransfer = TextTransfer.getInstance();
	
	private Transfer[] transferTypes = new Transfer[] {
			textTransfer
	};

	public Integer getWeight() {
		return new Integer(25);
	}
	
	public List<Task> createTasksFromDropEvent(DropTargetEvent event) {
		for (TransferData transferData : event.dataTypes) {
			if (textTransfer.isSupportedType(transferData)) {
				final String text = (String) textTransfer.nativeToJava(transferData);
				if (text == null)return null;
				return createTasks(text);
			}
		}
		return null;
	}

	public Transfer[] getSupportedTransfers() {
		return transferTypes;
	}

	private List<Task> createTasks(String text) {
		Task task = new Task();
		
		task.setSubject(inferSubject(text));
		task.setDetails(text);
		
		List<Task> tasks = new ArrayList<Task>();
		tasks.add(task);
		return tasks;
	}

	private String inferSubject(String text) {
		return StringUtils.truncateToNearestFullWord(text, 10, 30);
	}

}
