/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.task;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class Activator extends AbstractUIPlugin {

	//public static final String PLUGIN_ID = "org.eclipse.organizer.task.ui";
	
	//The shared instance.
	private static Activator plugin;

	public static final String IMAGE_CHECKED = "/icons/checked.gif";
	public static final String IMAGE_UNCHECKED = "/icons/unchecked.gif";
	public static final String IMAGE_CHECKED_HEADER = "/icons/header.gif";
	
	public static final String IMAGE_TASK = "/icons/task.gif";
	public static final String IMAGE_NEW_TASK = "/icons/new_task.gif";

	public static final String IMAGE_REFRESH = "/icons/refresh.gif";

	public static final String IMAGE_LOW = "/icons/low.gif";
	public static final String IMAGE_NORMAL = "/icons/normal.gif";
	public static final String IMAGE_HIGH = "/icons/high.gif";
	
	/**
	 * The constructor.
	 */
	public Activator() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}
		
	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 *
	 * @return the shared instance.
	 */
	public static Activator getDefault() {
		return plugin;
	}

	public static void logError(Exception e) {
		logError(new Status(IStatus.ERROR, getSymbolicName(), 0, e.getMessage(), e));
	}

	public static void logError(IStatus status) {
		getDefault().getLog().log(status);
	}

	public static void logError(String message) {
		logError(new Status(IStatus.ERROR, getSymbolicName(), 0, message, null));
	}

	public Image getImage(String path) {
		Image image = getImageRegistry().get(path);
		if (image == null) {
			ImageDescriptor descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(getBundle().getSymbolicName(), path);
			getImageRegistry().put(path, descriptor);
			image = getImageRegistry().get(path);
		}
		return image;
	}
	
	public ImageDescriptor getImageDescriptor(String path) {
		ImageDescriptor descriptor = getImageRegistry().getDescriptor(path);
		if (descriptor == null) {
			descriptor = AbstractUIPlugin.imageDescriptorFromPlugin(getBundle().getSymbolicName(), path);
			getImageRegistry().put(path, descriptor);
		}
		return descriptor;
	}

	public static String getSymbolicName() {
		return getDefault().getBundle().getSymbolicName();
	}

}
