/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.task.editors;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.ui.datepicker.DatePickerCombo;
import org.eclipse.samples.organizer.ui.task.Activator;
import org.eclipse.samples.organizer.ui.util.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;

public class TaskEditor extends EditorPart {
	private Label dueDateLabel = null;

	private DatePickerCombo dueDatePickerCombo = null;

	private Label subjectLabel = null;

	private Text subjectText = null;

	private Label priorityLabel = null;

	private Combo priorityCombo = null;

	private Label reminderLabel = null;

	private DatePickerCombo reminderDatePickerCombo = null;

	private Button completedCheckBox = null;

	private Text detailsText = null;

	private Task task;

	private boolean dirty = false;

	private Listener changeListener = new Listener() {
		public void handleEvent(Event event) {
			setDirty(true);
		}
	};

	@Override
	public void createPartControl(Composite parent) {
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		gridLayout.horizontalSpacing = 5;
		parent.setLayout(gridLayout);

		createSubjectLabel(parent);
		createSubjectText(parent);

		createDueDateLabel(parent);
		createDueDatePickerCombo(parent);

		createPriorityLabel(parent);
		createPriorityCombo(parent);

		createReminderDateLabel(parent);
		createReminderDatePickerCombo(parent);

		createCompletedCheckbox(parent);

		createDetailsText(parent);
	}

	private void createReminderDateLabel(Composite parent) {
		reminderLabel = new Label(parent, SWT.NONE);
		reminderLabel.setText("Reminder date:");
	}

	private void createPriorityLabel(Composite parent) {
		priorityLabel = new Label(parent, SWT.NONE);
		priorityLabel.setText("Priority:");
	}

	private void createDueDateLabel(Composite parent) {
		dueDateLabel = new Label(parent, SWT.NONE);
		dueDateLabel.setText("Due date:");
	}

   private void createSubjectLabel(Composite parent) {
		subjectLabel = new Label(parent, SWT.NONE);
		subjectLabel.setText("Subject:");
	}

	private void createSubjectText(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;

		subjectText = new Text(parent, SWT.BORDER);
		subjectText.setLayoutData(gridData);

		subjectText.setText(task.getSubject());

		subjectText.addListener(SWT.Modify, changeListener);
/*
		PlatformUI.getWorkbench().getHelpSystem()
			.setHelp(subjectText, "org.eclipse.organizer.task.ui.subjectField");*/
	}

	private void createCompletedCheckbox(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = GridData.END;

		completedCheckBox = new Button(parent, SWT.CHECK);
		completedCheckBox.setText("Completed");
		completedCheckBox.setLayoutData(gridData);

		//completedCheckBox.setSelection(task.isComplete());

	//	completedCheckBox.addListener(SWT.Selection, changeListener);

		PlatformUI.getWorkbench().getHelpSystem()
			.setHelp(completedCheckBox,	"org.eclipse.organizer.task.ui.completedField");

	}

	private void createDetailsText(Composite parent) {
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 5;
		gridData.grabExcessVerticalSpace = true;

		detailsText = new Text(parent, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
				| SWT.BORDER);
		detailsText.setLayoutData(gridData);

	//	detailsText.setText(task.getDetails() != null ? task.getDetails() : "");

	//	detailsText.addListener(SWT.Modify, changeListener);
		
		PlatformUI.getWorkbench().getHelpSystem()
			.setHelp(detailsText, "org.eclipse.organizer.task.ui.detailField");
	}

	/**
	 * This method initializes dueDatePickerCombo
	 * 
	 */
	private void createDueDatePickerCombo(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;

		dueDatePickerCombo = new DatePickerCombo(parent, SWT.BORDER);
		dueDatePickerCombo.setLayoutData(gridData);

		dueDatePickerCombo.setDate(task.getDueDate());

		dueDatePickerCombo.addListener(SWT.Modify, changeListener);

	//	PlatformUI.getWorkbench().getHelpSystem()
	//		.setHelp(dueDatePickerCombo, "org.eclipse.organizer.task.ui.dueDateField");
	}

	/**
	 * This method initializes priorityCombo
	 * 
	 */
	private void createPriorityCombo(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = false;

		priorityCombo = new Combo(parent, SWT.READ_ONLY);
		priorityCombo.setLayoutData(gridData);
		// TODO Probably inappropriate to use literal value.
		//priorityCombo
		//		.setItems(new String[] { Task.Priority.LOW.toString(),
		//				Task.Priority.NORMAL.toString(),
		//				Task.Priority.HIGH.toString() });

	//	priorityCombo.setText(task.getPriority().toString());

	//	priorityCombo.addListener(SWT.Selection, changeListener);

	//	PlatformUI.getWorkbench().getHelpSystem()
	//		.setHelp(priorityCombo, "org.eclipse.organizer.task.ui.priorityField");
}

	/**
	 * This method initializes reminderDatePickerCombo
	 * 
	 */
	private void createReminderDatePickerCombo(Composite parent) {
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;

		reminderDatePickerCombo = new DatePickerCombo(parent, SWT.BORDER);
		reminderDatePickerCombo.setLayoutData(gridData);

		reminderDatePickerCombo.setDate(task.getReminderDate());

		reminderDatePickerCombo.addListener(SWT.Modify, changeListener);

		PlatformUI.getWorkbench().getHelpSystem()
			.setHelp(reminderDatePickerCombo, "org.eclipse.organizer.task.ui.reminderDateField");
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Sort out aggregation of change notification.
		// boolean deliver = task.eDeliver();
		// task.eSetDeliver(false);
		task.setSubject(subjectText.getText());
		task.setComplete(completedCheckBox.getSelection());
		task.setDetails(detailsText.getText());
		task.setDueDate(dueDatePickerCombo.getDate());
		task.setReminderDate(reminderDatePickerCombo.getDate());
		task.setPriority(Task.Priority.valueOf(priorityCombo.getText()));
		// task.eSetDeliver(deliver);
		setDirty(false);
	}

	@Override
	// Not supported by this editor.
	public void doSaveAs() {
	}

	protected void setDirty(boolean dirty) {
		this.dirty = dirty;
		firePropertyChange(PROP_DIRTY);
	}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setPartName(getEditorTitle(input));
		setTitleImage(Activator.getDefault().getImage(Activator.IMAGE_TASK));
		setSite(site);
		setInput(input);
		task = ((TaskEditorInput) input).getTask();
	}

	private String getEditorTitle(IEditorInput input) {
		return StringUtils.truncateToNearestFullWord(input.getName(), 8, 15);
	}

	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void setFocus() {
		dueDatePickerCombo.setFocus();
	}

}
