package org.eclipse.samples.organizer.ui.task.helper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.exceptions.BackendException;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.tasks.filters.TaskFilter;
import org.eclipse.samples.organizer.ui.model.FeatureHelper;
import org.eclipse.samples.organizer.ui.task.dnd.TaskDropTarget;
import org.eclipse.samples.organizer.ui.task.wizards.ImportTasksWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.forms.HyperlinkGroup;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;

public class TaskViewFeatureHelper implements FeatureHelper {

	private FormToolkit toolkit;
	private Button filterComplete;

	public void createSwitcherContentArea(final Composite parent) {
		parent.setLayout(new FillLayout());
		Composite content = new Composite(parent, SWT.NONE);
		content.setLayout(new GridLayout(1, true));
		content.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		
		toolkit = new FormToolkit(parent.getDisplay());
		toolkit.getHyperlinkGroup().setHyperlinkUnderlineMode(HyperlinkGroup.UNDERLINE_HOVER);
		
		Hyperlink createTask = toolkit.createHyperlink(content, "Create task...", SWT.NONE);
		createTask.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		createTask.addHyperlinkListener(new HyperlinkAdapter() {
			public void linkActivated(HyperlinkEvent e) {
				createTask(parent.getShell());
			}			
		});
		
		Hyperlink refreshTask = toolkit.createHyperlink(content, "Refresh list", SWT.NONE);
		refreshTask.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
			
		Label separator = toolkit.createSeparator(content, SWT.HORIZONTAL);
		separator.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		filterComplete = toolkit.createButton(content, "Hide complete tasks", SWT.CHECK);
		filterComplete.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		filterComplete.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				TaskFilter.instance.setHideCompleteTasks(filterComplete.getSelection());
			}
		});
		final PropertyChangeListener filterCompleteChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateFilterComplete();
			}
		};
		TaskFilter.instance.addPropertyChangeListener(TaskFilter.HIDE_COMPLETE, filterCompleteChangeListener);

		filterComplete.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				TaskFilter.instance.removePropertyChangeListener(TaskFilter.HIDE_COMPLETE, filterCompleteChangeListener);
			}			
		});
		
		updateFilterComplete();
		
		
		content.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				toolkit.dispose();
			}			
		});
	}

	protected void updateFilterComplete() {
		filterComplete.setSelection(TaskFilter.instance.isHideCompleteTasks());		
	}

	private void createTask(final Shell shell) {
		final ArrayList<Task> tasks = new ArrayList<Task>();
		Task newTask = new Task();
		newTask.setSubject("New Task");
		tasks.add(newTask);
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ImportTasksWizard wizard = new ImportTasksWizard(tasks);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				dialog.open();
			}
		});
	}

	public void configureSwitcherButton(Button button) {
		new TaskDropTarget(button);
	}

	public String getSummary() {
		return String.valueOf(getCount()) + " tasks";
	}

	private int getCount() {
		try {
			return OrganizerManager.instance.count(TaskFilter.instance);
		} catch (BackendException e) {
			return 0;
		}
	}
}
