/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.task.wizards;

import java.text.DateFormat;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.ui.datepicker.DatePickerCombo;
import org.eclipse.samples.organizer.ui.datepicker.DateSelectedEvent;
import org.eclipse.samples.organizer.ui.datepicker.DateSelectionListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

public class ImportTaskWizardPage extends WizardPage {

	private final Task task;
	
	public ImportTaskWizardPage(Task task) {
		super(task.getSubject());
		this.task = task;
		setTitle(task.getSubject());
		setDescription("Provide information about this task");	
	}

	public void createControl(Composite parent) {
		Composite composite =  new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(3, false));

		createDueDateLabel(composite);		
		createDueDatePicker(composite);		
		createCompleteCheckbox(composite);	
		createReminderDateLabel(composite);		
		createReminderDatePicker(composite);	
		createFiller(composite);
		createSubjectLabel(composite);		
		createSubjectField(composite);
		createDetailsLabel(composite);		
		createDetailsField(composite);
		
		setControl(composite);		
	}

	private void createDueDateLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Due date:");
	}

	private void createFiller(Composite composite) {
		new Label(composite, SWT.NONE);
	}
	
	private void createDueDatePicker(Composite composite) {
	    final DatePickerCombo dueDatePicker = new DatePickerCombo(composite, SWT.BORDER);
		dueDatePicker.setDateFormat(DateFormat.LONG);
		dueDatePicker.setDate(task.getDueDate());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 1;
		dueDatePicker.setLayoutData(gridData);
		dueDatePicker.addDateSelectionListener(new DateSelectionListener() {
			public void dateSelected(DateSelectedEvent event) {
				task.setDueDate(event.date);				
			}
		});		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(dueDatePicker, "org.eclipse.samples.organizer.ui.task.dueDateField");

	}
	private void createReminderDateLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Reminder date:");
	}

	private void createReminderDatePicker(Composite composite) {
		final DatePickerCombo reminderDatePicker = new DatePickerCombo(composite, SWT.BORDER);
		reminderDatePicker.setDateFormat(DateFormat.LONG);
		reminderDatePicker.setDate(task.getReminderDate());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 1;
		reminderDatePicker.setLayoutData(gridData);
		reminderDatePicker.addDateSelectionListener(new DateSelectionListener() {
			public void dateSelected(DateSelectedEvent event) {
				task.setReminderDate(event.date);
			}
		});
		PlatformUI.getWorkbench().getHelpSystem().setHelp(reminderDatePicker, "org.eclipse.samples.organizer.ui.task.reminderDateField");
	}

	private void createCompleteCheckbox(Composite composite) {
		final Button completeButton = new Button(composite, SWT.CHECK);
		completeButton.setText("Complete");
		completeButton.setSelection(task.isComplete());
		completeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				task.setComplete(completeButton.getSelection());
			};
		});
	}

	private void createSubjectLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Subject:");
	}

	private void createSubjectField(Composite composite) {
		final Text subjectField = new Text(composite, SWT.BORDER);
		subjectField.setText(task.getSubject());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		subjectField.setLayoutData(gridData);
		subjectField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				task.setSubject(subjectField.getText());
				setTitle(subjectField.getText());
			}
		});
		addDropListener(subjectField);		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(subjectField, "org.eclipse.samples.organizer.ui.task.subjectField");

	}

	private void createDetailsLabel(Composite composite) {
		Label label = new Label(composite, SWT.NONE);
		label.setText("Details:");
		GridData gridData = new GridData();
		gridData.horizontalSpan = 3;
		label.setLayoutData(gridData);
	}

	private void createDetailsField(Composite composite) {
		final Text detailsField = new Text(composite, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER);
		detailsField.setText(task.getDetails() == null ? "" : task.getDetails());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 3;
		gridData.widthHint = 300;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.heightHint = 300;
		detailsField.setLayoutData(gridData);
		detailsField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				task.setDetails(detailsField.getText());
			}
		});
		addDropListener(detailsField);
		PlatformUI.getWorkbench().getHelpSystem().setHelp(detailsField, "org.eclipse.samples.organizer.ui.task.detailField");
	}
	
	private void addDropListener(final Text widget) {
		DropTarget dropTarget = new DropTarget(widget, DND.DROP_COPY | DND.DROP_DEFAULT);
		dropTarget.setTransfer(new Transfer[] {TextTransfer.getInstance()});
		dropTarget.addDropListener(new DropTargetAdapter() {
			TextTransfer textTransfer = TextTransfer.getInstance();

			public void dragEnter(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOperationChanged(DropTargetEvent event) {
				if (event.detail == DND.DROP_DEFAULT)
					event.detail = DND.DROP_COPY;
			}

			public void dragOver(DropTargetEvent event) {
			}

			public void drop(DropTargetEvent event) {

				if (textTransfer.isSupportedType(event.currentDataType)) {
					String text = (String) textTransfer.nativeToJava(event.dataTypes[1]);
					if (text != null) {
						widget.setText(text);
					}
				}
			}
		});
	}
	
	public IWizardPage getNextPage() {
		return getWizard().getNextPage(this);
	}

	public boolean canFlipToNextPage() {
		return getNextPage() != null;
	}
}

