/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.task.wizards;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.samples.organizer.OrganizerManager;
import org.eclipse.samples.organizer.backend.Backend;
import org.eclipse.samples.organizer.models.Task;
import org.eclipse.samples.organizer.ui.task.Activator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

public class ImportTasksWizard extends Wizard implements INewWizard {
			
	private final List<Task> tasks;
	private Backend backend;
	private List<Backend> backends;

	public ImportTasksWizard() {
		this(Collections.singletonList(getNewTask()));
	}

	public ImportTasksWizard(List<Task> tasks) {
		setWindowTitle("Import Tasks");
		this.tasks = tasks;
		this.backends = getBackends();
	}
	
	public static void importTasks(final Shell shell, final ArrayList<Task> tasks) {
		if (tasks.isEmpty()) return;
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ImportTasksWizard wizard = new ImportTasksWizard(tasks);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				dialog.create();
				dialog.open();
			}
		});
	}
	@Override
	public Image getDefaultPageImage() {
		return Activator.getDefault().getImage("/icons/tasks48.png");
	}
	
	public void addPages() {
		if (backends.isEmpty()) {
			// TODO If no backends are available, add a page that explains the problem (and nothing else).
			return;
		}
		this.backend = getDefaultBackend();
		if (backends.size() > 1) {
			addPage(new ImportTaskMainWizardPage(backends, backend));
		}
		if (tasks.size() > 10) {
			addPage(new ImportManyTasksWizardPage(tasks));
		} else {
			for (Task task : tasks) {
				addPage(new ImportTaskWizardPage(task));
			}
		}
	}

	private Backend getDefaultBackend() {
		assert backends != null;
		assert backends.size() > 0;
		
		Backend preferred = OrganizerManager.instance.getPreferredTaskBackend();
		if (preferred == null) return backends.get(0);
		return preferred;
	}

	private List<Backend> getBackends() {
		return OrganizerManager.instance.getBackends();
	}

	public boolean canFinish() {
		return backend != null;
	}
	
	public boolean performFinish() {
		OrganizerManager.instance.setPreferredTaskBackend(backend);
		try {
			OrganizerManager.instance.add(backend, tasks);
		} catch (Exception e) {
			showMessage("An error occurred while attempting to complete the operation. More information has be written to the log file.");
			Activator.logError(e);
		}
		return true;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {		
	}
	
	private void showMessage(String message) {
		MessageDialog.openInformation(getShell(), "Import Tasks Wizard", message);
	}

	public void setBackend(Backend backend) {
		this.backend = backend;		
	}

	private static Task getNewTask() {
		Task task = new Task();
		task.setSubject("New Task");
		return task;
	}
}
