/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.datepicker;

import java.util.Date;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;

public class DateCellEditor extends CellEditor {

	private DatePickerCombo datePicker;

	private Date value;

	public DateCellEditor(Table table, int dateFormat) {		
		super(table);
		datePicker.setDateFormat(dateFormat);
	}
	
	protected Control createControl(Composite composite) {
		datePicker = new DatePickerCombo(composite, SWT.NONE);
		datePicker.setFont(composite.getFont());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 1;
		datePicker.setLayoutData(gridData);
		datePicker.addDateSelectionListener(new DateSelectionListener() {
			public void dateSelected(DateSelectedEvent event) {
				value = event.date;
				valueChanged(true, true);
			}
		});
		datePicker.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                DateCellEditor.this.focusLost();
            }
        });
		
		return datePicker;
	}

	protected Object doGetValue() {
		return value;
	}

	protected void doSetFocus() {
		datePicker.setFocus();
	}

	protected void doSetValue(Object value) {
		this.value = (Date) value;
		datePicker.setDate(this.value);
	}
}
