/*******************************************************************************
 * Copyright (c) 2006 The Eclipse Foundation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    The Eclipse Foundation - initial API and implementation
 *******************************************************************************/
package org.eclipse.samples.organizer.ui.util;

public class StringUtils {
	/**
	 * <p>This method returns a potentially shortened version of a string.
	 * The returned value will contain at most <code>maximumChars</code>
	 * and at least <code>minimumChars</code> of the original content. 
	 * The truncated string will contain full words from the original. 
	 * That is, the truncation will not occur in the middle of a word. 
	 * If the original string is truncated, "..." is appended to the
	 * result.</p>
	 * 
	 * <p>If a null <code>value</code> is provided an empty string is
	 * returned.</p>
	 *  
	 * @param value the original value.
	 * @param minimumChars the minimum size of the truncated string.
	 * @param maximumChars the maximum size of the truncated string.
	 * @return
	 */
	public static String truncateToNearestFullWord(String original, int minimumChars, int maximumChars) {
		if (original == null) return "";
		original = original.trim();
		String value = original;
		value = value.split("\n")[0].trim();
		if (value.length() > maximumChars) {
			int index=maximumChars-3; // Subtract 3 to account for possible appending of "..."
			// Scan backwards until whitespace is found
			while (index-->minimumChars) {
				if (Character.isWhitespace(value.charAt(index))) break;
			}
			// Keep scanning backwards until a letter or digit is encountered
			while (index>minimumChars) {
				if (Character.isLetterOrDigit(value.charAt(index))) break;
				index--;
			}
			value = value.substring(0, index + 1);
		}
		
		if (value.length() < original.length()) value = value + "...";
		
		return value;
	}
}
